/******************************************************************************
 *
 * CHECK_HTTP.C
 *
 * Program: HTTP plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/06/07 20:51:10 $
 * 1999-10-27 Ethan Galstad
 *  Added -nohtml option to suppress HTML link in output
 * 1999-09-23 Modified by Ian Cass, Knowledge Matters Ltd
 *  Now returns a http link the the web page its monitoring, so you
 *  can see for yourself if it's working.
 * 2000-01-25 Modified by Karl DeBisschop, Information Please
 *  Searches page contents for regex, checks redirects
 *
 * Command line: CHECK_HTTP <host_address> [-e expect] [-a auth]
 *                          [-hn host_name] [-p port] [-u url]
 *                          [-wt warn_time] [-ct crit_time] [-to to_sec]
 *
 * Description:
 *
 * This plugin will attempt to open an HTTP connection with the host.
 * Successul connects return STATE_OK, refusals and timeouts return
 * STATE_CRITICAL, other errors return STATE_UNKNOWN.  Successful
 * connects, but incorrect reponse messages from the host result in
 * STATE_WARNING return values.  If you are checking a virtual server
 * that uses "host headers" you must supply the FQDN (fully qualified
 * domain name) as the [host_name] argument.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "../common/version.h"
#include "netutils.h"
#include "utils.h"

#define HTTP_PORT       80
#define HTTP_EXPECT     "HTTP/1."
#define HTTP_URL        "/"

#define MAX_PAGE_SIZE 16384

#define REGS 0
#define MAX_RE_SIZE 256
#ifdef HAVE_REGEX_H
#include <regex.h>
regex_t preg;
/* regmatch_t pmatch[REGS]; */
char re[MAX_RE_SIZE];
char errbuf[MAX_INPUT_BUFFER];
int cflags=REG_NOSUB|REG_EXTENDED|REG_NEWLINE;
int errcode;
#endif

time_t start_time,end_time;
char timestamp[10]="";
int server_port=HTTP_PORT;
char server_port_text[MAX_HOST_ADDRESS_LENGTH];
char server_address[MAX_HOST_ADDRESS_LENGTH];
char host_name[MAX_HOST_ADDRESS_LENGTH];
char server_url[MAX_INPUT_BUFFER]=HTTP_URL;
char server_type[MAX_INPUT_BUFFER]="http";
char server_expect[MAX_INPUT_BUFFER]=HTTP_EXPECT;
char string_expect[MAX_INPUT_BUFFER]="";
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
char user_auth[MAX_INPUT_BUFFER]="";
int display_html=TRUE;
int onredirect=STATE_OK;

void print_usage(char *);
void print_help(char *);
int process_arguments(int, char **);
static char *base64(char *bin, int len);
void check_http (void);

int main(int argc, char **argv){

  if(process_arguments(argc,argv)!=OK){
    printf("Incorrect arguments supplied\n\n");
    print_revision(argv[0],"$Revision: 1.1 $");
    print_usage(argv[0]);
    return STATE_UNKNOWN;
  }

  if(strstr(timestamp,":")){
    if(strstr(server_url,"?"))
      sprintf(server_url,"%s&%s",server_url,timestamp);
    else
      sprintf(server_url,"%s?%s",server_url,timestamp);
  }

  if(display_html==TRUE)
    printf("<A HREF=\"http://%s:%d%s\" target=\"_blank\">",host_name,server_port,server_url);

  /* initialize alarm signal handling, set socket timeout, start timer */
  signal(SIGALRM,socket_timeout_alarm_handler);
  alarm(socket_timeout);
  start_time = time(NULL);

  check_http();
  return STATE_UNKNOWN;

}


/* process command-line arguments */
int process_arguments(int argc, char **argv){
  int x;
  time_t t;
  struct tm *tstruct;

  /* no options were supplied */
  if(argc<2)
    return ERROR;

  /* first option is always the server name/address */
  strncpy(server_address,argv[1],sizeof(server_address)-1);
  server_address[sizeof(server_address)-1]=0;

  /* set the host name to the server address (until its overridden) */
  strcpy(host_name,server_address);

  /* process all remaining arguments */
  for(x=3;x<=argc;x++){

    /* we got the string to expect from the server */
    if(!strcmp(argv[x-1],"-e") || 
       !strcmp(argv[x-1],"--expect")){
      if(x<argc){
	strncpy(server_expect,argv[x],sizeof(server_expect)-1);
	server_expect[sizeof(server_expect)-1]=0;
	x++;
      }
      else
	return ERROR;
    }

    /* we got the URL to check */
    else if(!strcmp(argv[x-1],"-u") || 
	    !strcmp(argv[x-1],"--url")){
      if(x<argc){
	strncpy(server_url,argv[x],sizeof(server_url)-1);
	server_url[sizeof(server_url)-1]=0;
	x++;
      }
      else
	return ERROR;
    }

    /* we go the host name to use in the host header */
    else if(!strcmp(argv[x-1],"-hn") || 
	    !strcmp(argv[x-1],"-h") || 
	    !strcmp(argv[x-1],"--hostname")){
      if(x<argc){
	strncpy(host_name,argv[x],sizeof(host_name)-1);
	host_name[sizeof(host_name)-1]=0;
	x++;
      }
      else
	return ERROR;
    }

    /* we got the port number to use */
    else if(!strcmp(argv[x-1],"-p") || 
	    !strcmp(argv[x-1],"--port")){
      if(x<argc){
	server_port=atoi(argv[x]);
	x++;
      }
      else
	return ERROR;
    }

    /* we got the socket timeout */
    else if(!strcmp(argv[x-1],"-to")||
	    !strcmp(argv[x-1],"-t")||
	    !strcmp(argv[x-1],"--timeout")){
      if(x<argc){
	socket_timeout=atoi(argv[x]);
	if(socket_timeout<=0)
	  return ERROR;
	x++;
      }
      else
	return ERROR;
    }

    /* we got the warning threshold time */
    else if(!strcmp(argv[x-1],"-wt")||
	    !strcmp(argv[x-1],"-w")||
	    !strcmp(argv[x-1],"--warning")){
      if(x<argc){
	warning_time=atoi(argv[x]);
	check_warning_time=TRUE;
	x++;
      }
      else
	return ERROR;
    }

    /* we got the critical threshold time */
    else if(!strcmp(argv[x-1],"-ct")||
	    !strcmp(argv[x-1],"-c")||
	    !strcmp(argv[x-1],"--critical")){
      if(x<argc){
	critical_time=atoi(argv[x]);
	check_critical_time=TRUE;
	x++;
      }
      else
	return ERROR;
    }

    /* we got the user's authorization info */
    else if(!strcmp(argv[x-1],"-a")||
	    !strcmp(argv[x-1],"--authinfo")){
      if(x<argc){
	strncpy(user_auth,argv[x],sizeof(user_auth)-1);
	user_auth[sizeof(user_auth)-1]=0;
	x++;
      }
      else
	return ERROR;
    }

    /* we got the no HTML option */
    else if(!strcmp(argv[x-1],"-nohtml")||
	    !strcmp(argv[x-1],"--nohtml")){
      display_html=FALSE;
    }

    /* we got the timestamp option */
    else if(!strcmp(argv[x-1],"--mark")){
      t = time(NULL);
      tstruct = localtime(&t);
      strftime(timestamp,10,"%T",tstruct);
    }

    /* we got the  string option */
    else if(!strcmp(argv[x-1],"-str")||
	    !strcmp(argv[x-1],"-s")||
	    !strcmp(argv[x-1],"--string")){
      if(x<argc){
	strncpy(string_expect,argv[x],sizeof(string_expect)-1);
	string_expect[sizeof(string_expect)-1]=0;
	x++;
      }
      else
	return ERROR;
    }

#ifdef HAVE_REGEX_H
    /* we got the regex option */
    else if(!strcmp(argv[x-1],"-r")||
	    !strcmp(argv[x-1],"--ereg")||
	    !strcmp(argv[x-1],"--eregi")){
      if(x<argc){
	if(strlen(argv[x])>=MAX_RE_SIZE-1){
	  printf("HTTP UNKNOWN: regex too long\n");
	  exit(STATE_UNKNOWN);
	}
	strcpy(re,argv[x]);
	if(!strcmp(argv[x-1],"--eregi"))
	  cflags|=REG_ICASE;
	errcode=regcomp(&preg,re,cflags);
	if(errcode!=0){
	  regerror(errcode,&preg,errbuf,MAX_INPUT_BUFFER);
	  printf("Could Not Compile Regular Expression: %s",errbuf);
	  return ERROR;
	}
	x++;
      }
      else
	return ERROR;
    }
#endif

    /* we got an onredirect option */
    else if(!strcmp(argv[x-1],"--onredirect")){
      if(x>=argc)
	return ERROR;
      if(!strcmp(argv[x],"follow"))
	onredirect=STATE_DEPENDENT;
      if(!strcmp(argv[x],"unknown"))
	onredirect=STATE_UNKNOWN;
      if(!strcmp(argv[x],"ok"))
	onredirect=STATE_OK;
      if(!strcmp(argv[x],"warning"))
	onredirect=STATE_WARNING;
      if(!strcmp(argv[x],"critical"))
	onredirect=STATE_CRITICAL;
      x++;
    }

    /* else we got something else... */
    else
      return ERROR;
  }

  return OK;
}



/* written by lauri alanko */
static char * base64(char *bin, int len){

  char *buf= (char *)malloc((len+2)/3*4+1);
  int i=0, j=0;

  char BASE64_END = '=';
  char base64_table[64]= "ABCDEFGHIJKLMNOPQRSTUVWXYZ" 
    "abcdefghijklmnopqrstuvwxyz"
    "0123456789+/";
        
  while( j < len - 2 ) {
    buf[i++]=base64_table[bin[j]>>2];
    buf[i++]=base64_table[((bin[j]&3)<<4)|(bin[j+1]>>4)];
    buf[i++]=base64_table[((bin[j+1]&15)<<2)|(bin[j+2]>>6)];
    buf[i++]=base64_table[bin[j+2]&63];
    j+=3;
  }

  switch ( len - j ) {
  case 1:
    buf[i++] = base64_table[bin[j]>>2];
    buf[i++] = base64_table[(bin[j]&3)<<4];
    buf[i++] = BASE64_END;
    buf[i++] = BASE64_END;
    break;
  case 2:
    buf[i++] = base64_table[bin[j] >> 2];
    buf[i++] = base64_table[((bin[j] & 3) << 4) 
			   | (bin[j + 1] >> 4)];
    buf[i++] = base64_table[(bin[j + 1] & 15) << 2];
    buf[i++] = BASE64_END;
    break;
  case 0:
    break;
  }

  buf[i]='\0';
  return buf;
}



void check_http (void){
  int sd;
  char buffer[MAX_INPUT_BUFFER];
  char msg[MAX_INPUT_BUFFER];
  char *status_line;
  char *header;
  char *page;
  char *auth;
  size_t i=0;
  size_t pagesize=0;
  char *full_page=NULL;
  char *pos=NULL;

  /* try to connect to the host at the given port number */
  if(my_tcp_connect(server_address,server_port,&sd)!=STATE_OK){
    snprintf(msg,MAX_INPUT_BUFFER-1,"Unable to open TCP socket");
    terminate(STATE_CRITICAL,msg);
  }

  /* we connected, so close connection before exiting */
  /* send the GET request */
  sprintf(buffer,"GET %s HTTP/1.0\r\n",server_url);
  send(sd,buffer,strlen(buffer),0);

  /* optionally send the host header info */
  if(strcmp(host_name,"")){
    sprintf(buffer,"Host: %s\r\n",host_name);
    send(sd,buffer,strlen(buffer),0);
  }

  /* optionally send the authentication info */
  if(strcmp(user_auth,"")){
    auth = base64(user_auth, strlen(user_auth));
    sprintf(buffer,"Authorization: Basic %s\r\n",auth);
    send(sd,buffer,strlen(buffer),0);
  }

  /* send a newline so the server knows we're done with the request */
  sprintf(buffer,"\r\n\r\n\r\n");
  send(sd,buffer,strlen(buffer),0);
  
  /* read in page from server */
  pagesize=(size_t)0;
  while ( (i=recv(sd,buffer,MAX_INPUT_BUFFER,0)) > 0 &&
	  (pos = realloc(full_page,pagesize+i)) != NULL ) {
    full_page=pos;
    strncpy(full_page+pagesize,buffer,i);
    pagesize += i;
  }
  pagesize++;
  pos = realloc(full_page,pagesize);
  full_page=pos;
  pos[pagesize]=0; 

  /* return a CRITICAL status if we couldn't read any data */
  if(pagesize==(size_t)1){
    snprintf(msg,MAX_INPUT_BUFFER-1,"No data received from host %s",timestamp);
    terminate(STATE_CRITICAL,msg);
  }

  /* close the connection */
  close(sd);

  /* reset the alarm */
  alarm(0);

  /* leave full_page untouched so we can free it later */
  page=full_page;

  /* find status line and null-terminate it */
  status_line=page;
  page+=(size_t)strcspn(page,"\r\n");
  pos=page;
  page+=(size_t)strspn(page,"\r\n");
  status_line[pos-status_line]=0;

  /* find header info and null terminate it */
  header=page;
  while(strpbrk(page,":")<strpbrk(page,"\r\n")){
    page+=(size_t)strcspn(page,"\r\n");
    pos=page;
    page+=(size_t)strspn(page,"\r\n");
  }
  header[pos-header]=0;
/*    printf("%s\n",header); */

  /* check elapsed time */
  end_time = time(NULL);
  snprintf(msg,MAX_INPUT_BUFFER-1,"HTTP problem: %s - %d second response time %s%s",status_line,(int)(end_time-start_time),timestamp,(display_html?"</A>":""));
  if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
    terminate(STATE_CRITICAL,msg);
  if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
    terminate(STATE_WARNING,msg);

  /* make sure the status line matches the response we are looking for */
  if(!strstr(status_line,server_expect)){
    if(server_port==HTTP_PORT)
      snprintf(msg,MAX_INPUT_BUFFER-1,"Invalid HTTP response received from host\n");
    else
      snprintf(msg,MAX_INPUT_BUFFER-1,"Invalid HTTP response received from host on port %d\n",server_port);
    terminate(STATE_CRITICAL,msg);
  }

  /* check the return code */
  /* server errors result in a critical state */
  if(strstr(status_line,"500")||strstr(status_line,"501")||strstr(status_line,"502")||strstr(status_line,"503"))
    terminate(STATE_CRITICAL,msg);

  /* client errors result in a warning state */
  if(strstr(status_line,"400")||strstr(status_line,"401")||strstr(status_line,"402")||strstr(status_line,"403")||strstr(status_line,"404"))
    terminate(STATE_WARNING,msg);

  /* check redirected page if specified */
  if(strstr(status_line,"300")||strstr(status_line,"301")||strstr(status_line,"302")||strstr(status_line,"303")||strstr(status_line,"304")){
    if(onredirect==STATE_DEPENDENT){
      pos=header;
      while(pos){
	if(sscanf(pos,"%*[Ll]%*[Oo]%*[Cc]%*[Aa]%*[Tt]%*[Ii]%*[Oo]%*[Nn]: %[HTPhtp]://%[a-zA-Z0-9.-]:%[0-9]%[/a-zA-Z0-9._-]",server_type,server_address,server_port_text,server_url)==4){
	  strcpy(host_name,server_address);
	  server_port=atoi(server_port_text);
	  check_http();
	}else if(sscanf(pos,"%*[Ll]%*[Oo]%*[Cc]%*[Aa]%*[Tt]%*[Ii]%*[Oo]%*[Nn]: %[HTPhtp]://%[a-zA-Z0-9.-]%[/a-zA-Z0-9._-]",server_type,server_address,server_url)==3){
	  strcpy(host_name,server_address);
	  server_port=80;
	  check_http();
	}else if(sscanf(pos,"%*[Ll]%*[Oo]%*[Cc]%*[Aa]%*[Tt]%*[Ii]%*[Oo]%*[Nn]: %[HTPhtp]://%[a-zA-Z0-9.-]:%[0-9]",server_type,server_address,server_port_text)==3){
	  strcpy(host_name,server_address);
	  strcpy(server_url,"/");
	  server_port=atoi(server_port_text);
	  check_http();
	}else if(sscanf(pos,"%*[Ll]%*[Oo]%*[Cc]%*[Aa]%*[Tt]%*[Ii]%*[Oo]%*[Nn]: %[HTPhtp]://%[a-zA-Z0-9.-]",server_type,server_address)==2){
	  strcpy(host_name,server_address);
	  server_port=80;
	  strcpy(server_url,"/");
	  check_http();
	}
	pos+=(size_t)strcspn(pos,"\r\n");
	pos+=(size_t)strspn(pos,"\r\n");
      }
      printf("HTTP UNKNOWN: Could not find redirect location - %s%s",
	     status_line,
	     (display_html?"</A>":""));
      exit(STATE_UNKNOWN);
    }
    else if(onredirect==STATE_UNKNOWN)
      printf("HTTP UNKNOWN");
    else if(onredirect==STATE_OK)
      printf("HTTP ok");
    else if(onredirect==STATE_WARNING)
      printf("HTTP WARNING");
    else if(onredirect==STATE_CRITICAL)
      printf("HTTP CRITICAL");
    snprintf(msg,MAX_INPUT_BUFFER-1,
	     ": %s - %d second response time %s%s",
	     status_line,
	     (int)(end_time-start_time),
	     timestamp,
	     (display_html?"</A>":""));
    terminate(onredirect,msg);
  }

  /* these checks should be last */

  if (strlen(string_expect)){
    if(strstr(page,string_expect)){
      printf("HTTP ok: %s - %d second response time %s%s\n",
	     status_line,
	     (int)(end_time-start_time),
	     timestamp,
	     (display_html?"</A>":""));
      exit(STATE_OK);
    }else{
      printf("HTTP CRITICAL: string not found%s\n",
	     (display_html?"</A>":""));
      exit(STATE_CRITICAL);
    }
  }

#ifdef HAVE_REGEX_H
  if (strlen(re)){
    errcode=regexec(&preg,page,(size_t)REGS,NULL,0);
    if(errcode==0){
      printf("HTTP ok: %s - %d second response time %s%s\n",
	     status_line,
	     (int)(end_time-start_time),
	     timestamp,
	     (display_html?"</A>":""));
      exit(STATE_OK);
    }else{
      if(errcode==REG_NOMATCH){
	printf("HTTP CRITICAL: pattern not found%s\n",
	       (display_html?"</A>":""));
	exit(STATE_CRITICAL);
      }else{
	regerror(errcode,&preg,errbuf,MAX_INPUT_BUFFER);
	printf ("Execute Error: %s\n",errbuf);
	exit(STATE_CRITICAL);
      }
    }
  }
#endif

  /* We only get here if all tests have been passed */
  snprintf(msg,MAX_INPUT_BUFFER-1,
	   "HTTP ok: %s - %d second response time %s%s",
	   status_line,
	   (int)(end_time-start_time),
	   timestamp,
	   (display_html?"</A>":""));
  terminate(STATE_OK,msg);
}

void print_usage(char *command_name){
  char *plugin_name;
  plugin_name=my_basename(command_name);
  printf("Usage: %s <host> [-u <uri>] [-p <port>] [-a auth] [-w <warn time>] [-c <critical time>] [-t <timeout>] [-e <expect>] [-r regex] [--ereg <case-senstive regex> | --eregi <case-insensitive regex>] [--nohtml]\n\n",plugin_name);
}

void print_help(char *command_name){
  printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
  printf("License: GPL\n\n");
  print_usage(command_name);
  printf("\n");
  printf("Options:\n");
  printf(" -e [expect]    = String to expect in first line of server response (default: \"%s\")\n",HTTP_EXPECT);
  printf(" -s [string]    = String to expect in the content\n");
  printf(" -u [url]       = URL to GET (default: /)\n");
  printf(" -p [port]      = Port number (default: %d)\n",HTTP_PORT);
  printf(" -h [host_name] = Host name argument to GET command - used for servers using host headers\n");
  printf(" -w [warn_time] = Response time to result in warning status (seconds)\n");
  printf(" -c [crit_time] = Response time to result in critical status (seconds)\n");
  printf(" -t [to_sec]    = Seconds before connection times out (default: %d)\n",DEFAULT_SOCKET_TIMEOUT);
  printf(" -a [auth]      = Username:password for sites requiring basic authentication\n");
  printf(" [--nohtml]     = Don't allow HTML in the plugin output\n\n");
  printf("This plugin attempts to contact the HTTP service on the specified host. If\n");
  printf("possible, supply numeric address for the host, as this will bypass DNS lookup.\n");
}
