/******************************************************************************
 *
 * CHECK_HTTPS.C
 *
 * Program: HTTPS plugin for NetSaint
 * License: GPL
 * Copyright (c) 2000 Rene Klootwijk <rene@klootwijk.org>
 * (bits and pieces are taken from check_http, written by Ethan Galstad)
 *
 * Last modified: 02/15/2000 19:50:00 $revision$
 *
 * Command line: CHECK_HTTPS <host_address> [-p port] [-u url] [-c days]
 *                           [-t warning_time] [-n] [-V] [-h]
 *
 * Description:
 * This plugin connects to a server running an SSL enabled web server.
 * Depending on the command line arguments it will then check the ability
 * of the server to serve content or it will check if the certificate is valid.
 * When the server is not able to serve the content, a STATE_CRITICAL or 
 * STATE_WARNING will be returned depending on the return code of the server. 
 * When the certificate is expired a STATE_CRITICAL will be returned, if the
 * certificate is valid shorter than the specified period, a STATE_WARNING will
 * be returned. In all other cases a STATE_OK will be returned.
 *
 * License Information:
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#ifdef HAVE_SSL_H
  #include <rsa.h>
  #include <crypto.h>
  #include <x509.h>
  #include <pem.h>
  #include <ssl.h>
  #include <err.h>
#else
  #include <openssl/rsa.h>
  #include <openssl/crypto.h>
  #include <openssl/x509.h>
  #include <openssl/pem.h>
  #include <openssl/ssl.h>
  #include <openssl/err.h>
#endif

#define REVISION     "$revision$"

#define HTTPS_PORT   443
#define HTTP_EXPECT  "HTTP/1."
#define HTTP_URL     "/"


time_t       start_time,end_time;
int          port = HTTPS_PORT;
char         server_address[MAX_HOST_ADDRESS_LENGTH] = "";
char         server_name[MAX_HOST_ADDRESS_LENGTH];
char         url[MAX_INPUT_BUFFER]=HTTP_URL;
int          warning_time = 0;
int          check_warning_time = FALSE;
int          critical_time = 0;
int          check_critical_time = FALSE;
int          check_cert = FALSE;
int          days_till_exp;
int          display_html = TRUE;
int          sd;
SSL_CTX*     ctx;
SSL*         ssl;

void usage(char*);
void help(void);
int  process_cl_parms (int,char**);
int  connect_SSL(void);
int  check_certificate (X509**);
int  get_url(void);


int main(int argc,char **argv) {
  int   result;
  X509* server_cert;


  /* Process command line parameters */
  if (process_cl_parms(argc,argv) != OK)
    exit(STATE_UNKNOWN);

  /* Make an SSL connection */
  if (connect_SSL() != OK)
    exit(STATE_CRITICAL);

  /* Get certificate */
  if ((server_cert = SSL_get_peer_certificate (ssl)) != NULL) {
    if (check_cert == TRUE)
      result = check_certificate(&server_cert);
    else
      result = get_url();
 
    /* Release certificate */ 
    X509_free(server_cert);
  } else {
    printf("ERROR: Cannot retrieve server certificate.\n");
    result = STATE_CRITICAL;
  }
  
  /* Send SSL/TLS close_notify */
  SSL_shutdown(ssl);

  /* Close the connection and free resources */
  close(sd);
  SSL_free(ssl);
  SSL_CTX_free(ctx);

  /* Reset the alarm */
  alarm(0);

  exit(result);
}


int process_cl_parms (int argc,char **argv) {
  int  c;

  /* Parse command line parameters */
  if (argc < 2) {
    printf("ERROR: No command line parameters specified.\n");
    usage(argv[0]);
    return ERROR;
  }

  /* The first argument should be the address or name of the host to query */
  if (argv[1][0] != '-') {
    strncpy(server_address,argv[1],sizeof(server_address) - 1);
    server_address[sizeof(server_address) - 1] = '\0';
    if (is_host(server_address) == FALSE) {
      printf("ERROR: Invalid host name or address specified.\n");
      usage(argv[0]);
      return ERROR;
    }
    strcpy(server_name,server_address);

    /* Process arguments starting with argument 2 */
    optind++;
  }

  /* Process the command line parameters */ 
  while ((c = getopt(argc,argv,"Vp:c:u:t:hn")) != EOF) {
    switch (c) {
      case 'V':
        print_revision(argv[0],REVISION);
        return ERROR;
        break;

      case 'p':
        port = atoi(optarg);
        break;

      case 'c':
        days_till_exp = atoi(optarg);
        check_cert = TRUE;
        break;

      case 'u':
        strncpy(url,optarg,sizeof(url) - 1);
        url[sizeof(url) - 1] = '\0';
        break;

      case 't':
        warning_time = atoi(optarg);
        check_warning_time = TRUE;
        break;

      case 'n':
        display_html = FALSE;
        break;

      case 'h':
        usage(argv[0]);
        help();
        return ERROR;

      case '?':
        usage(argv[0]);
        return ERROR;

      default:
        printf("ERROR: Unknown error.\n");
        usage(argv[0]);
        return ERROR;
    }
  }

  if(!strlen(server_address)) {
    printf("ERROR: No host address or name specified!\n");
    usage(argv[0]);
    return ERROR;
  }
  
  return OK;
}


int connect_SSL(void) {
  SSL_METHOD  *meth;

  /* Initialize SSL context */
  SSLeay_add_ssl_algorithms();
  meth = SSLv2_client_method();
  SSL_load_error_strings();
  if ((ctx = SSL_CTX_new (meth)) == NULL) {
    printf("ERROR: Cannot create SSL context.\n");
    return STATE_CRITICAL;
  }

  /* Initialize alarm signal handling */
  signal(SIGALRM,socket_timeout_alarm_handler);

  /* Set socket timeout */
  alarm(socket_timeout);

  /* Save start time */
  time(&start_time);

  /* Make TCP connection */
  if (my_tcp_connect(server_address,port,&sd) == STATE_OK) {
    /* Do the SSL handshake */
    if ((ssl = SSL_new(ctx)) != NULL) {
      SSL_set_fd(ssl,sd);
      if (SSL_connect(ssl) != -1)
        return OK;
      ERR_print_errors_fp(stderr);

    } else 
      printf("ERROR: Cannot initiate SSL handshake.\n");
   
    

    SSL_free(ssl);
  }

  close(sd);
  SSL_CTX_free (ctx);
  
  return STATE_CRITICAL;
}


int check_certificate (X509** server_cert) {
  ASN1_STRING* tm;
  int          offset;
  struct tm    stamp;
/*    int          result = STATE_OK; */
  int          days_left;
  char         timestamp[14];


  /* Retrieve timestamp of certificate */
  tm = X509_get_notAfter(*server_cert);

  /* Generate tm structure to process timestamp */
  if (tm->type == V_ASN1_UTCTIME) {
    if (tm->length < 10) {
      printf("ERROR: Wrong time format in certificate.\n");
      return STATE_CRITICAL;
    } else {
      stamp.tm_year = (tm->data[0] - '0') * 10 + (tm->data[1] - '0');
      if (stamp.tm_year < 50)
        stamp.tm_year += 100;
      offset = 0;
    }
  } else {
    if (tm->length < 12) {
      printf("ERROR: Wrong time format in certificate.\n");
      return STATE_CRITICAL;
    } else {
      stamp.tm_year = (tm->data[0] - '0') * 1000 + (tm->data[1] - '0') * 100 + \
                      (tm->data[2] - '0') * 10 + (tm->data[3] - '0');
      stamp.tm_year -= 1900;
      offset = 2;
    }
  }
  stamp.tm_mon   = (tm->data[2 + offset] - '0') * 10 + \
                   (tm->data[3 + offset] - '0') - 1;
  stamp.tm_mday  = (tm->data[4 + offset] - '0') * 10 + \
                   (tm->data[5 + offset] - '0');
  stamp.tm_hour  = (tm->data[6 + offset] - '0') * 10 + \
                   (tm->data[7 + offset] - '0');
  stamp.tm_min   = (tm->data[8 + offset] - '0') * 10 + \
                   (tm->data[9 + offset] - '0');
  stamp.tm_sec   = 0;
  stamp.tm_isdst = -1;

  days_left = (mktime(&stamp) - time(NULL)) / 86400;
  sprintf(timestamp,"%02d/%02d/%04d %02d:%02d",stamp.tm_mon + 1,stamp.tm_mday, \
          stamp.tm_year + 1900,stamp.tm_hour,stamp.tm_min);
  
  if (days_left > 0 && days_left <= days_till_exp) {
    printf("Certificate expires in %d day(s) (%s).\n",days_left,timestamp);
    return STATE_WARNING;
  }
  if (days_left < 0 ) {
    printf("Certificate expired on %s.\n",timestamp);
    return STATE_CRITICAL;
  }
  
  if (days_left == 0 ) {
    printf("Certificate expires today (%s).\n",timestamp);
    return STATE_WARNING;
  }

  printf("Certificate will expire on %s.\n",timestamp);

  return STATE_OK;
}


int get_url(void) {
  char  buffer[MAX_INPUT_BUFFER];
  char* status_line;
  int   result;
  int   response_time;

  /* Send the GET request */
  sprintf(buffer,"GET %s HTTP/1.0\r\n\r\n",url);
  if (SSL_write (ssl,buffer,strlen(buffer)) == -1) {
    ERR_print_errors_fp(stderr);
    return STATE_CRITICAL;
  }
 
  if (SSL_read (ssl,buffer,MAX_INPUT_BUFFER - 1) == -1) {
    ERR_print_errors_fp(stderr);
    return STATE_CRITICAL;
  }

  /* Make sure we find the response we are looking for */
  if (!strstr(buffer,HTTP_EXPECT)) {
    if (port == HTTPS_PORT)
      printf("Invalid HTTP response received from host\n");
    else
      printf("Invalid HTTP response received from host on port %d\n",port);
    return STATE_CRITICAL;
  }

  /* Save end time */
  time(&end_time);
  response_time = (int)(end_time - start_time);

  /* Get first line */
  status_line = (char *)strtok(buffer,"\r\n");
 
  result = STATE_OK;

  if (strstr(status_line,"200"))
    result=STATE_OK;
  else if (strstr(status_line,"201"))
    result = STATE_OK;
  else if (strstr(status_line,"202"))
    result = STATE_OK;
  else if (strstr(status_line,"203"))
    result=STATE_OK;
  else if(strstr(status_line,"204"))
    result=STATE_OK;
   
  /* We should be checking the redirected page, but we don't */
  else if(strstr(status_line,"301"))
    result=STATE_OK;
  else if(strstr(status_line,"302"))
    result=STATE_OK;
  else if(strstr(status_line,"303"))
    result=STATE_OK;
  else if(strstr(status_line,"304"))
    result=STATE_OK;

  /* Client errors result in a warning state */
  else if(strstr(status_line,"400"))
    result=STATE_WARNING;
  else if(strstr(status_line,"401"))
    result=STATE_WARNING;
  else if(strstr(status_line,"402"))
    result=STATE_WARNING;
  else if(strstr(status_line,"403"))
    result=STATE_WARNING;
  else if(strstr(status_line,"404"))
    result=STATE_WARNING;

  /* Server errors result in a critical state */
  else if(strstr(status_line,"500"))
    result=STATE_CRITICAL;
  else if(strstr(status_line,"501"))
    result=STATE_CRITICAL;
  else if(strstr(status_line,"502"))
    result=STATE_CRITICAL;
  else if(strstr(status_line,"503"))
    result=STATE_CRITICAL;

  else result=STATE_UNKNOWN;

  if (result == STATE_OK) {
   
    /* Is response within specified time? */ 
    if (check_warning_time == TRUE && response_time > warning_time) 
      result = STATE_WARNING;

    if (display_html == TRUE) 
      printf("<A HREF='https://%s:%d%s' target='_blank'>",server_name,port,url);
 
    printf("HTTPS OK [%s] - Response Time %s [%ds]",status_line, \
           (result == STATE_WARNING)?"WARNING":"OK",response_time);

    if (display_html == TRUE) 
      printf("</A>");

    printf("\n");
  

  } else
    printf("HTTPS ERROR [%s]\n",status_line);

  return result;
}


void usage(char* cmd) {
  printf("\nUsage: %s <host> [-p port] [-u URL] [-t time] [-c days] [-n] [-V] "
         "[-h]\n\n", cmd);
  printf("       port, the port number to use [Default = 443]\n"
         "       URL,  the url to query for [Default = '/']\n"
         "       time, max. response time (in sec.), when exceeded return\n"
         "             state WARNING\n"
         "       days, min. number of days a certificate has to be valid.\n"
         "             Note: When this option is used only the certificate\n"
         "                   will be checked.\n"
         "       -n,   suppress html output\n"
         "       -V,   display version information of this plugin\n"
         "       -h,   display a verbose help message\n\n");
}


void help(void) {
  printf("       This plugin checks whether an SSL enabled web server is able\n"
         "       to serve content (optionally within a specified time) or\n"
         "       whether the X509 certificate is still valid for the\n"
         "       specified number of days.\n\n"
         "       Check ability to serve content:\n"
         "       ===============================\n"
         "       Example: check_https www.verisign.com -t 5\n"
         "       When the 'www.verisign.com' server returns its content\n"
         "       within 5 seconds, a STATE_OK will be returned. When the\n"
         "       server returns its content but exceeds the 5 seconds\n"
         "       threshold, a STATE_WARNING will be returned. When an error\n"
         "       occurs, a STATE_CRITICAL will be returned.\n\n"
         "       Check validity of certificate:\n"
         "       ==============================\n"
         "       Example: check_https www.verisign.com -c 14\n"
         "       When the certificate of 'www.verisign.com' is valid for more\n"
         "       than 14 days, a STATE_OK will be returned. When the\n"
         "       certificate is still valid but less than 14 days, a\n"
         "       STATE_WARNING will be returned. A STATE_CRITICAL will be\n"
         "       returned when the certificate is expired.\n\n");
}
