/******************************************************************************
 *
 * CHECK_PROCESS.C
 *
 * Program: Process plugin for NetSaint
 * License: GPL
 * Copyright (c) 2000 Rene Klootwijk (rene@klootwijk.org)
 *
 * Last Modified: $Date: 2000/06/14 20:33:54 $
 *
 * Command line: CHECK_PROCESS <limit> [-s state] [-u user] [-p ppid] [-c cmd]
 *
 * Description:
 *
 * This plugin determines the current entries in the process table and uses
 * the command line parameters state, user, ppid and cmd to filter the list.
 * The low, high and exact parameters are used to check on the number of
 * matches. At least one limit has to be specified using the low, high or
 * exact parameters. 
 *
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "popen.h"
#include "utils.h"

/* Constants used to compute a checksum */
#define STATE    1
#define USER     2
#define PPID     4
#define CMD      8
#define HIGH     16
#define LOW      32
#define EXACT    64

void print_usage(char *);
void print_help(char *);

void print_usage(char *cmd){
  printf(" Help: %s -H\n"
	 "Usage: %s <limit> [-s state] [-u user] [-p ppid] [-c cmd]\n"
	 ,cmd,cmd);
}

void print_help(char *cmd) {
  print_revision(cmd,"$Revision: 1.1 $");
  printf("Copyright (c) 2000, Rene Klootwijk (rene@klootwijk.org)\n"
	 "License: GPL\n\n");
  print_usage(cmd);
  printf("\nwhere:\n\n"
	 " limit = the limit(s) that determine when the plugin returns an OK or CRITICAL\n"
	 "         state. Limits are specified using the following parameters:\n"
	 "           -l <n>   minimum number of processes that should match the criteria.\n"
	 "           -h <n>   maximum number of processes that should match the criteria.\n"
	 "           -e <n>   exact number of processes that should match the criteria.\n"
	 "         (Note: -e cannot be used together with -l and/or -h!)\n"
	 " state = state of the process (O,S,R,Z or T) (default = all)\n"
	 " user  = effective user ID (textual) of the process (default = all)\n"
	 " ppid  = ID of the parent process (default = all)\n"
	 " cmd   = a string to identify the process. This string must be a part of\n"
	 "        the cmd line as displayed in the ps command (default = all)\n\n"
	 "This plugin determines the current entries in the process table and uses the\n"
	 "command line parameters state, user, ppid and cmd to filter the list. The low,\n"
	 "high and exact parameters are used to check on the number of matches. At least\n"
	 "one limit has to be specified using the low, high or exact parameters.\n");
}


int main(int argc, char **argv){
  char input_buffer[MAX_INPUT_BUFFER];
  char tmp_buffer[MAX_INPUT_BUFFER];
  char cmd[MAX_INPUT_BUFFER];
  char proc_cmd[MAX_INPUT_BUFFER];
  FILE *fp;
  int ppid=0, proc_ppid;
  char user[9],proc_user[9];
  char state[6], proc_state[6];
  char state_options[] = "RSTZD";
  int i, c;
  int high = 0;
  int low = 0;
  int exact = 0;
  int checksum = 0;
  int options = 0;
  int resultsum, matched;
  pid_t my_ppid;


  /* Check if for any command line parameters */ 
  if (argc == 1) {
    printf("ERROR: No parameters specified!\n\n");
    print_usage(my_basename(argv[0]));
    exit(STATE_UNKNOWN);
  }
  
  /* Process command line parameters */
  for(c=1;c<argc;c++)
    if(strcmp("-to",argv[c])==0)
      strcpy(argv[c],"-t");
  while((c = getopt(argc, argv, "VHs:u:p:c:h:l:e:")) != EOF) {
    switch(c) {
      case 's':
        strncpy(state,optarg,sizeof(state) - 1);
        state[sizeof(state) - 1] = 0;
        for(i = 0 ; i < strlen(state) ; i++) {
          if(strchr(state_options,state[i]) == NULL) {
            printf("ERROR: Invalid process state specified!\n\n");
            print_usage(my_basename(argv[0]));
            exit(STATE_UNKNOWN);
          }
        }
        options |= STATE;
        checksum |= STATE;
        break;

      case 'u':
        strncpy(user,optarg,sizeof(user) - 1);
        user[sizeof(user) - 1] = 0;
        options |= USER;
        checksum |= USER;
        break;

      case 'p':
        ppid = atoi(optarg);
        options |= PPID;
        checksum |= PPID;
        break;

      case 'c':
        strncpy(cmd,optarg,sizeof(cmd) - 1);
        cmd[sizeof(cmd) - 1] = 0;
        options |= CMD;
        checksum |= CMD;
        break;

      case 'h':
        high = atoi(optarg);
        options |= HIGH;
        break;

      case 'l':
        low = atoi(optarg);
        options |= LOW;
        break;

      case 'e':
        exact = atoi(optarg);
        options |= EXACT;
        break;

      case 'V': /* version */
	print_revision(my_basename(my_basename(argv[0])),"$Revision: 1.1 $");
	exit(STATE_OK);

      case 'H': /* help */
	print_help(my_basename(argv[0]));
	exit(STATE_OK);

      default:
        printf("ERROR: Invalid parameter specified!\n\n");
        print_usage(my_basename(argv[0]));
        exit(STATE_UNKNOWN);
    }
  }
  if((options & EXACT) && ((options & LOW) || (options & HIGH))) {
    printf("ERROR: -e cannot be defined together with -h and/or -l\n\n");
    print_usage(my_basename(argv[0]));
    exit(STATE_UNKNOWN);
  }
  if(!((options & EXACT) || (options & LOW) || (options & HIGH))) {
    printf("ERROR: No limits are specified (-e, -l or -h)!\n");
    print_usage(my_basename(argv[0]));
    exit(STATE_UNKNOWN);
  }

  /* Run the PS command */
  fp=spopen(PS_RAW_COMMAND);
  if(fp==NULL){
    printf("ERROR: Could not open pipe: %s\n",PS_COMMAND);
    return STATE_UNKNOWN;
  }

  /* Obtain the ID of the parent process */
  my_ppid = getppid();

  /* Ignore the header line */
  fgets(input_buffer,MAX_INPUT_BUFFER-1,fp);

  /* Process process table */
  matched = 0;
  while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){
    resultsum = 0;
    /* Determine column values */
    strcpy(tmp_buffer,input_buffer);
    strncpy(proc_state,strtok(tmp_buffer," "),sizeof(proc_state) - 1);
    proc_state[sizeof(proc_state) - 1] = 0;
    strncpy(proc_user,strtok(NULL," "),sizeof(proc_user) - 1);
    proc_user[sizeof(proc_user) - 1] = 0;
    proc_ppid  = atoi(strtok(NULL," "));
    strncpy(proc_cmd,strtok(NULL,"@"),sizeof(proc_cmd) - 1);
    proc_cmd[sizeof(proc_cmd) - 1] = 0;

    /* Do not process this process (the calling process) */
    if(proc_ppid == my_ppid) { continue; }

    /* Process column values */
    if(options & STATE) {
      if(strstr(state,proc_state) != NULL) { resultsum |= STATE; }
    }
    if(options & USER) {
      if(strcmp(user,proc_user) == 0) { resultsum |= USER; }
    }
    if (options & CMD) {
      if(strstr(proc_cmd,cmd) != NULL) { resultsum |= CMD; }
    }
    if(options & PPID) {
      if(ppid == proc_ppid) { resultsum |= PPID; }
    }
    if(checksum == resultsum) { matched++; }
  }
  /* close the pipe */
  spclose(fp);

  /* Determine the return message and code of this plugin */ 
  if ((matched < low) && (options & LOW)) {
    printf("Too less processes matched the specified criteria (%d).\n",matched);
    exit(STATE_CRITICAL);
  }
  if ((matched > high) && (options & HIGH)) {
    printf("Too many processes matched the specified criteria (%d).\n",matched);
    exit(STATE_CRITICAL);
  }
  if ((matched != exact) && (options & EXACT)) {
    printf("The number of processes that matched the specified criteria is not equal to the specified number (%d).\n",matched);
    exit(STATE_CRITICAL);
  }

  printf("%d processes matched the specified criteria.\n",matched);
  exit(STATE_OK);
}
