#! /bin/sh
#
# Log file pattern detector plugin for NetSaint
# Written by Ethan Galstad (netsaint@linuxbox.com)
# Last Modified: 07-31-1999
#
# Usage: ./check_log <log_file> <old_log_file> <pattern>
#
# Description:
#
# This plugin will scan a log file (specified by the <log_file> option)
# for a specific pattern (specified by the <pattern> option).  Successive
# calls to the plugin script will only report *new* pattern matches in the
# log file, since an copy of the log file from the previous run is saved
# to <old_log_file>.
#
# Output:
#
# On the first run of the plugin, it will return an OK state with a message
# of "Log check data initialized".  On successive runs, it will return an OK
# state if *no* pattern matches have been found in the *difference* between the
# log file and the older copy of the log file.  If the plugin detects any 
# pattern matches in the log diff, it will return a CRITICAL state and print
# out a message is the following format: "(x) last_match", where "x" is the total
# number of pattern matches found in the file and "last_match" is the last 
# entry in the log file which matches the pattern.
#
# Notes:
#
# If you use this plugin make sure to keep the following in mind:
#
#    1.  The "max_attempts" value for the service should be 1, as this
#        will prevent NetSaint from retrying the service check (the
#        next time the check is run it will not produce the same results).
#
#    2.  The "notify_recovery" value for the service should be 0, so that
#        NetSaint does not notify you of "recoveries" for the check.  Since
#        pattern matches in the log file will only be reported once and not
#        the next time, there will always be "recoveries" for the service, even
#        though recoveries really don't apply to this type of check.
#
#    3.  You *must* supply a different <old_file_log> for each service that
#        you define to use this plugin script - even if the different services
#        check the same <log_file> for pattern matches.  This is necessary because
#        of the way the script operates.
#
# Examples:
#
# Check for login failures in the syslog...
#
#        ./check_log /var/log/messages ./check_log.badlogins.old "LOGIN FAILURE"
#
# Check for port scan alerts generated by Psionic's PortSentry software...
#
#        ./check_log /var/log/message ./check_log.portscan.old "attackalert"
#

# Paths to commands used in this script.  These
# may have to be modified to match your system setup.

PATH="/usr/bin:/bin"

echocmd="echo"
grepcmd="grep"
diffcmd="diff"
tailcmd="tail"
catcmd="cat"
rmcmd="rm"

# The temporary file that the script should use while
# processing the log file.

tempdiff="/usr/local/netsaint/test/var/check_log.tmp"


# Make sure the correct number of command line
# arguments have been supplied

if [ $# -ne 3 ]; then
	$echocmd "Incorrect arguments supplied\n"
	exit 2
fi

# Grab the command line arguments

logfile=$1
oldlog=$2
query=$3

# If the source log file doesn't exist, exit

if [ ! -e $logfile ]; then
	$echocmd "Log check error: Log file $logfile does not exist!\n"
	exit 2
fi

# If the old log file doesn't exist, this must be the first time
# we're running this test, so copy the original log file over to
# the old diff file and exit

if [ ! -e $oldlog ]; then
	$catcmd $logfile > $oldlog
	$echocmd "Log check data initialized...\n"
	exit 0
fi

# The old log file exists, so compare it to the original log now

$diffcmd $logfile $oldlog > $tempdiff

# Count the number of matching log entries we have
count=`$grepcmd -c "$query" $tempdiff`

# Get the last matching entry in the diff file
lastentry=`$grepcmd "$query" $tempdiff | $tailcmd --lines=1`

# If we couldn't find any matches, exit with no error
if [ "$count" = "0" ]; then
	$echocmd "Log check ok - 0 pattern matches found\n"
	$catcmd $logfile > $oldlog
	$rmcmd -f $tempdiff
	exit 0
fi

# Print the total matches found, as well as the last entry we found
$echocmd "($count) $lastentry"

$catcmd $logfile > $oldlog
$rmcmd -f $tempdiff

exit 1


