/******************************************************************************
 *
 * CHECK_BY_SSH.C
 *
 * Program: SSH Check plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)
 *
 * Last Modified: $Date: 2000/06/12 18:32:33 $
 *
 * Command line: CHECK_BY_SSH [options] <user> <host> <command>
 *
 * Description:
 *
 * Runs a check on a remote host using ssh, wrapping the propoer
 * timeout around the ssh invocation
 * 
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "popen.h"
#include "utils.h"

int process_arguments(int, char **);
int call_getopt(int, char **);
int validate_arguments(void);
void print_help(char *command_name);
void print_usage(char *command_name);

char input_buffer[MAX_INPUT_BUFFER]="";
char command[MAX_INPUT_BUFFER]="";
char comm[MAX_INPUT_BUFFER]="";
char hostname[MAX_HOST_ADDRESS_LENGTH]="";

int verbose=FALSE;

int main(int argc, char **argv){
	int result=STATE_UNKNOWN;

	/* process arguments */
	if(process_arguments(argc,argv)==ERROR)
		{
			printf("%s: Could not parse arguments\n\n",my_basename(argv[0]));
			print_usage(my_basename(argv[0]));
			return STATE_UNKNOWN;
		}

	/* Set signal handling and alarm timeout */
	if(signal(SIGALRM,popen_timeout_alarm_handler)==SIG_ERR)
		{
			printf("Cannot catch SIGALRM");
			return STATE_UNKNOWN;
		}
	alarm(timeout_interval);
	
	if (verbose)
		printf("%s\n",comm);

	/* run the command */
	child_process=spopen(comm);
	if(child_process==NULL)
		{
			printf("Unable to open pipe: %s",comm);
			return STATE_UNKNOWN;
		}

	/* open STDERR	for spopen */
	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL)
		{
			printf("Could not open stderr for %s\n",PATH_TO_UPTIME);
		}

	/* WARNING if output found on stderr */
	if(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		{
			printf("%s\n",input_buffer);
			return STATE_WARNING;
		}
	(void)fclose(child_stderr);

	/* get first line only from remote command */
	fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process);

	/* close the pipe */
	result=spclose(child_process);

	/* print the first line from the remote command */
	printf("%s\n",input_buffer);

	/* return error status from remote command */
	return result;
}

void print_help(char *cmd){
	print_revision(cmd,"$Revision: 1.14 $");
	printf("Copyright (c) 1999	Karl DeBisschop (kdebisschop@alum.mit.edu)\n"
	 "License: GPL\n\n");
	print_usage(cmd);
	printf("\nArguments:\n"
	 " -f tells ssh to fork rather than create a tty\n"
	 " [timeout]	= specify timeout (default is %d) [optional]\n"
	 " [user]		 = SSH user name on remote host [optional]\n"
	 " [identity] = identity of an authorized key [optional]\n"
	 " <host>		 = Name of remote host\n"
	 " <command>	= Command to execute (up to 14 tokens)\n\n"
	 "This plugin will execute a command on a remote host using SSH\n\n",
	 DEFAULT_SOCKET_TIMEOUT);
}

void print_usage(char *cmd)
{
	printf("Usage:\n"
	 " %s [-f] [-t timeout] [-i identity] [-l user] -H <host> <command>\n"
	 " %s -V prints version info\n"
	 " %s -h prints more detailed help\n",
	 cmd,cmd,cmd);
}

/* process command-line arguments */
int process_arguments(int argc, char **argv)
{
	int c;
	
	if(argc<2)
		return ERROR;
	
	for(c=1;c<argc;c++)
		if(strcmp("-to",argv[c])==0)
			strcpy(argv[c],"-t");

	c=0;
	while(c+=(call_getopt(argc-c,&argv[c])))
		{
			if(argc <= c)
	break;
			if (hostname[0]==0)
	{
		if(strlen(argv[c])<MAX_HOST_ADDRESS_LENGTH)
			strcpy(hostname,argv[c]);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
	}
			else if (command[0]==0)
	{
		if(strlen(argv[c])<MAX_INPUT_BUFFER)
			strcpy(command,argv[c++]);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		for (; c<argc; c++)
			{
				if(strlen(command)+strlen(argv[c])<MAX_INPUT_BUFFER-1)
		sprintf(command,"%s %s",command,argv[c]);
				else
		terminate(STATE_UNKNOWN,"Input buffer overflow");
			}
	}
		}

	if(strlen(input_buffer)+strlen(hostname)<MAX_INPUT_BUFFER-1)
		sprintf(input_buffer,"%s %s",input_buffer,hostname);
	else
		terminate(STATE_UNKNOWN,"Input buffer overflow");
	if(strlen(SSH_COMMAND)+strlen(input_buffer)+strlen(command)
		 <MAX_INPUT_BUFFER-2)
		sprintf(comm,"%s %s %s",SSH_COMMAND,input_buffer,command);
	else
		terminate(STATE_UNKNOWN,"Input buffer overflow");

	return validate_arguments();
}

/* Call getopt */
int call_getopt(int argc, char **argv){
	int c,i=1;

#ifdef HAVE_GETOPT_H
		int option_index = 0;
		static struct option long_options[] =
		{ 
			{"version", no_argument,0,'V'},
			{"help",		no_argument,0,'h'},
			{"fork",		no_argument,0,'f'},
			{"timeout", required_argument,0,'t'},
			{"host",		required_argument,0,'H'},
			{"identity",required_argument,0,'i'},
			{"user",		required_argument,0,'u'},
			{"logname", required_argument,0,'l'},
			{"command", required_argument,0,'C'},
			{0,0,0,0}
		};
#endif

	while (1)
		{
#ifdef HAVE_GETOPT_H
			c = getopt_long(argc,argv,"+?Vhft:H:p:i:u:l:C:",long_options,&option_index);
#else
			c = getopt(argc,argv,"+?Vhft:H:p:i:u:l:C:");
#endif

			if(c==-1||c==EOF)
	break;

			i++;
			switch (c)
	{
	case 't':
	case 'H':
	case 'p':
	case 'i':
	case 'u':
	case 'l':
	case 'C':
		i++;
	}

			switch (c)
	{
	case '?': /* help */
		print_usage(my_basename(argv[0]));
		exit(STATE_UNKNOWN);
	case 'V': /* version */
		print_revision(my_basename(argv[0]),"$Revision: 1.14 $");
		exit(STATE_OK);
	case 'h': /* help */
		print_help(my_basename(argv[0]));
		exit(STATE_OK);
	case 'f': /* fork to background */
		if(strlen(input_buffer)<MAX_INPUT_BUFFER-3)
			sprintf(input_buffer,"%s -f",input_buffer);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		break;
	case 't': /* timeout period */
		if(!is_integer(optarg))
			{
				printf("%s: Timeout Interval must be an integer!\n\n",my_basename(argv[0]));
				print_usage(my_basename(argv[0]));
				exit(STATE_UNKNOWN);
			}
		timeout_interval = atoi(optarg);
		break;
	case 'H': /* host */
		if(strlen(optarg)<MAX_HOST_ADDRESS_LENGTH)
			strcpy(hostname,optarg);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		break;
	case 'i': /* identity */
		if(strlen(input_buffer)+strlen(optarg)<MAX_INPUT_BUFFER-4)
			sprintf(input_buffer,"%s -i %s",input_buffer,optarg);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		break;
	case 'u': /* user name */
		if(strlen(input_buffer)+strlen(optarg)<MAX_INPUT_BUFFER-4)
			sprintf(input_buffer,"%s -l %s",input_buffer,optarg);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		break;
	case 'l': /* login name */
		if(strlen(input_buffer)+strlen(optarg)<MAX_INPUT_BUFFER-4)
			sprintf(input_buffer,"%s -l %s",input_buffer,optarg);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		break;
	case 'C': /* Command for remote machine */
		if(strlen(optarg)<MAX_INPUT_BUFFER)
			strcpy(command,optarg);
		else
			terminate(STATE_UNKNOWN,"Input buffer overflow");
		break;
	}
		}
	return i;
}

int validate_arguments(void)
{
	if(command[0]==0 || hostname[0]==0)
		return ERROR;
	return OK;
}
