/******************************************************************************
 *
 * CHECK_DISK.C
 *
 * Program: Disk space plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/06/21 15:36:11 $
 *
 * Command line: CHECK_DISK <w_dfp> <c_dfp> <filesystem>
 *
 * Description:
 *
 * This plugin will use the /bin/df command to check the free space on
 * currently mounted filesystems.  If the percent used disk space is
 * above <c_dfp>, a STATE_CRITICAL is returned.  If the percent used
 * disk space is above <w_dfp>, a STATE_WARNING is returned.  If the
 * speicified filesystem cannot be read, a STATE_CRITICAL is returned,
 * other errors with reading the output result in a STATE_UNKNOWN
 * error.
 *
 * Notes:
 *  - IRIX support added by Charlie Cook 4-16-1999
 *  - Modifications by Karl DeBisschop 1999-11-24
 *     reformat code to 80 char screen width
 *     set STATE_WARNING if stderr is written or spclose status set
 *     set default result to STAT_UNKNOWN
 *     initailize usp to -1, eliminate 'found' variable
 *     accept any filename/filesystem
 *     use sscanf, drop while loop
 *
 *****************************************************************************/

#include "../common/common.h"
#include "popen.h"
#include "utils.h"

void help(char *);

void license(void);
void usage(char *);
int process_arguments(int, char **);
int call_getopt(int, char **);
int validate_arguments(void);
void print_help(char *command_name);
void print_usage(char *command_name);

int w_df=-1;
int c_df=-1;
float w_dfp=-1.0;
float c_dfp=-1.0;
char path[MAX_INPUT_BUFFER]="";

int main(int argc, char **argv){
	int usp=-1;
	int total_disk=-1;
	int used_disk=-1;
	int free_disk=-1;
	int result=STATE_UNKNOWN;
	char command_line[MAX_INPUT_BUFFER]="";
	char input_buffer[MAX_INPUT_BUFFER]="";
	char file_system[MAX_INPUT_BUFFER]="";
	char mount_point[MAX_INPUT_BUFFER]="";

	if(process_arguments(argc,argv)!=OK){
		print_usage(my_basename(argv[0]));
		exit(STATE_UNKNOWN);
	        }

	snprintf(command_line,MAX_INPUT_BUFFER-1,"%s %s",DF_COMMAND,path);

	child_process=spopen(command_line);
	if(child_process==NULL){
		printf("Could not open pipe: %s\n",command_line);
		return STATE_UNKNOWN;
	        }

	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL){
		printf("Could not open stderr for %s\n",command_line);
	        }

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process))
	  if(index(input_buffer,'/'))
	    continue;

	if(sscanf(input_buffer,"%s %d %d %d %d%% %s",file_system,&total_disk,&used_disk,&free_disk,&usp,mount_point)==6) {
	}
	else if (sscanf(input_buffer,"%s %*s %d %d %d %d%% %s",file_system,&total_disk,&used_disk,&free_disk,&usp,mount_point)==6) {
	}
	else {
		printf("Unable to read output:\n%s\n%s\n",command_line,input_buffer);
		return result;
	}

	/* If we get anything on stderr, at least set warning */
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		result=max(result,STATE_WARNING);

	/* close stderr */
	(void)fclose(child_stderr);

	/* close the pipe */
	if(spclose(child_process))
		result=max(result,STATE_WARNING);

        /* usp set, means we found the filesystem */
	/* check the percent used space against thresholds */
	if(usp>=0 && usp>=(100.0-c_dfp))
		result=STATE_CRITICAL;
	else if(c_df>=0 && free_disk<=c_df)
		result=STATE_CRITICAL;
	else if(usp>=0 && usp>=(100.0-w_dfp))
		result=STATE_WARNING;
	else if(w_df>=0 && free_disk<=w_df)
		result=STATE_WARNING;
	else if(usp>=0.0)
		result=STATE_OK;
	
	if(result==STATE_OK)
		terminate(STATE_OK,"Disk ok - %d kB (%d%%) free on %s\n",free_disk,100-usp,file_system);
	else if(usp<0)
		printf("Disk %s not mounted or nonexistant\n",argv[3]);
	else if(result==STATE_UNKNOWN)
		printf("Unable to read output\n%s\n%s\n",command_line,input_buffer);
	else /* STATE_WARNING or STATE_CRITICAL */
		printf("Only %d kB (%d%%) free on %s\n",free_disk,100-usp,file_system);

	return result;
        }


void print_usage(char *command_name){
	char *plugin_name;
	plugin_name=my_basename(command_name);
	printf("Usage: %s <w_usp> <c_usp> <filesystem>\n",plugin_name);
	printf("       %s [-p <path>] [-w <warn_KB> -c <crit_KB>] [-t <timeout>]\n",plugin_name);
	printf("       %s [-p <path>] [-w <warn_p>%% -c <crit_p>%%] [-t <timeout>]\n",plugin_name);
	printf("       %s [-p <path>] [-w <warn_KB>:<warn_p>%% -c <crit_KB>:<crit_p>%%] [-t <timeout>]\n",plugin_name);
}

void print_help(char *command_name){
	char *prog;
	prog=my_basename(command_name);
	print_revision(command_name,"$Revision: 1.8 $");
	printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n"
	       "License: GPL\n\n");
	print_usage(command_name);
	printf("\nOptions:\n"
	       " <warn_KB>  = KB of remaining space that will cause a WARNING state\n"
	       " <crit_KB>  = KB of remaining space that will cause a CRITICAL state\n"
	       " <warn_p> = Percent remaining space that will cause a WARNING state\n"
	       " <crit_p> = Percent remaining space that will cause a CRITICAL state\n"
	       "\n"
	       "This plugin will check the percent of used disk space on a mounted\n"
	       "file system and generate an alert if percentage is above one of the\n"
	       "threshold values.\n\n");
}

/* process command-line arguments */
int process_arguments(int argc, char **argv){
  int c;

  if(argc<2)
    return ERROR;

  for(c=1;c<argc;c++){
    if(strcmp("-to",argv[c])==0){
      strcpy(argv[c],"-t");
    }
  }

  c=0;
  while(c+=(call_getopt(argc-c,&argv[c]))){
    if(argc <= c)
      break;
    if (w_dfp==-1 && is_intnonneg(argv[c]))
      w_dfp=(100.0-atof(argv[c]));
    else if (c_dfp==-1 && is_intnonneg(argv[c]))
      c_dfp=(100.0-atof(argv[c]));
    else if (path[0]==0)
      strncpy(path,argv[c],MAX_INPUT_BUFFER);
  }

  return validate_arguments();
}

int call_getopt(int argc, char **argv){
  int c,i=1;

#ifdef HAVE_GETOPT_H
    int option_index = 0;
    static struct option long_options[] =
    { 
      {"warning",  required_argument,0,'w'},
      {"critical", required_argument,0,'c'},
      {"timeout",  required_argument,0,'t'},
      {"path",     required_argument,0,'p'},
      {"partition",required_argument,0,'p'},
      {"version",  no_argument,      0,'V'},
      {"help",     no_argument,      0,'h'},
      {0,0,0,0}
    };
#endif

  while (1){
#ifdef HAVE_GETOPT_H
    c = getopt_long(argc,argv,"+?Vht:c:w:p:",long_options,&option_index);
#else
    c = getopt(argc,argv,"+?ht:c:w:p:");
#endif

    if(c==-1||c==EOF)
      break;

    i++;
    switch (c)
      {
      case 't':
      case 'c':
      case 'w':
      case 'p':
	i++;
      }

    switch (c)
      {
      case 'w': /* warning time threshold */
	if (is_intnonneg(optarg)) {
	  w_df=atoi(optarg);
	  break;
	} else if (strstr(optarg,":") &&
		   strstr(optarg,"%") &&
		   sscanf(optarg,"%d:%f%%",&w_df,&w_dfp)==2) {
	  break;
	} else if (strstr(optarg,"%") &&
		   sscanf(optarg,"%f%%",&w_dfp)==1) {
	  break;
	} else {
	  printf("%s: Warning threshold must be integer or percentage!\n\n",my_basename(argv[0]));
	  print_usage(my_basename(argv[0]));
	  exit(STATE_UNKNOWN);
	}
      case 'c': /* critical time threshold */
	if (is_intnonneg(optarg)) {
	  c_df=atoi(optarg);
	  break;
	} else if (strstr(optarg,":") &&
		   strstr(optarg,"%") &&
		   sscanf(optarg,"%d:%f%%",&c_df,&c_dfp)==2) {
	  break;
	} else if (strstr(optarg,"%") &&
		   sscanf(optarg,"%f%%",&c_dfp)==1) {
	  break;
	} else {
	  printf("%s: Critical threshold must be integer or percentage!\n\n",my_basename(argv[0]));
	  print_usage(my_basename(argv[0]));
	  exit(STATE_UNKNOWN);
	}
      case 't': /* timeout period */
	if(is_integer(optarg)){
	  timeout_interval = atoi(optarg);
	  break;
	}else{
	  printf("%s: Timeout Interval must be an integer!\n\n",my_basename(argv[0]));
	  print_usage(my_basename(argv[0]));
	  exit(STATE_UNKNOWN);
	}
      case 'p': /* path or partition */
	strncpy(path,optarg,MAX_INPUT_BUFFER);
	break;
      case 'V': /* version */
	print_revision(my_basename(argv[0]),"$Revision: 1.8 $");
	exit(STATE_UNKNOWN);
      case 'h': /* help */
      case '?': /* help */
	print_help(my_basename(argv[0]));
	exit(STATE_UNKNOWN);
      }
  }
  return i;
}

int validate_arguments(){
  if(w_df<0 && c_df<0 && w_dfp<0 && c_dfp<0){
    printf("INPUT ERROR: Unable to parse command line\n");
    return ERROR;
  } else if((w_dfp>=0 || c_dfp>=0) &&
     (w_dfp<0 || c_dfp<0 || w_dfp>100 || c_dfp>100 || c_dfp>w_dfp)){
    printf("INPUT ERROR: <c_dfp> (%f) should be less than <w_dfp> (%f) and both should be between zero and 100 percent, inclusive\n",c_dfp,w_dfp);
    return ERROR;
  } else if ((w_df>=0 || c_df>=0) && (w_df<0 || c_df<0 || c_df>w_df)) {
    printf("INPUT ERROR: <c_df> (%d) should be less than <w_df> (%d) and both should be greater than zero\n",c_df,w_df);
    return ERROR;
  } else {
    return OK;
  }
}
