/******************************************************************************
 *
 * CHECK_LOAD.C
 *
 * Written by Felipe Gustavo de Almeida <galmeida@linux.ime.usp.br>
 * License: GPL
 * Command line: CHECK_LOAD <wload1> <cload1> <wload5> <cload5> <wload15> <cload15>
 * First Written: 04/17/99 
 *
 * Modifications:
 * 
 * 05/18/1999 - Modified to work getloadavg where available, and use uptime
 *		where neither proc or getloadavg are found.  Also use autoconf.
 *                 mods by Karl DeBisschop (kdebiss@alum.mit.edu)
 * 07/01/1999 - Added some #DEFINEs to allow compilation under NetBSD, as
 *		suggested by Andy Doran.
 *	           mods by Ethan Galstad (netsaint@linuxbox.com)
 * 07/17/1999 - Initialized la[] array to prevent NetBSD from complaining
 *		   mods by Ethan Galstad (netsaint@linuxbox.com)
 * 08/18/1999 - Integrated some code with common plugin utilities
 *		   mods by Ethan Galstad (netsaint@linuxbox.com)
 * $Date: 2000/05/03 03:11:02 $
 * Note: The load format is the same used by "uptime" and "w"
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "utils.h"

#ifdef HAVE_SYS_LOADAVG_H
#include <sys/loadavg.h>
#endif

/* needed for compilation under NetBSD, as suggested by Andy Doran */
#ifndef LOADAVG_1MIN
#define LOADAVG_1MIN	0
#define LOADAVG_5MIN	1
#define LOADAVG_15MIN	2
#endif	/* !defined LOADAVG_1MIN */

#include "popen.h"
#ifdef HAVE_PROC_LOADAVG

#endif

int main(int argc, char **argv)
{
#if HAVE_GETLOADAVG==1
  int result;
  double la[3]={0.0,0.0,0.0};  /* NetBSD complains about unitialized arrays */
#elif HAVE_PROC_LOADAVG==1
  FILE *fp;
  char input_buffer[MAX_INPUT_BUFFER];
  char *tmp_ptr;
#else
  int result;
  char input_buffer[MAX_INPUT_BUFFER];
#endif

  float wload1, wload5, wload15;
  float cload1, cload5, cload15;
  float la1, la5, la15;

  if(argc!=7){
    printf("Incorrect number of arguments supplied\n");
    printf("\n");
    print_revision(argv[0],"$Revision: 1.5 $");
    printf("Copyright (c) 1999 Felipe Gustavo de Almeida <galmeida@linux.ime.usp.br>\n");
    printf("\n");
    printf("Usage: %s <wload1> <cload1> <wload5> <cload5> <wload15> <cload15>\n",argv[0]);
    printf("\n");
    printf("Note:\n");
    printf(" <wload1>     = 1 minute load average necessary to result in a WARNING state\n");
    printf(" <cload1>     = 1 minute load average necessary to result in a CRITICAL state\n");
    printf(" <wload5>     = 5 minute load average necessary to result in a WARNING state\n");
    printf(" <cload5>     = 5 minute load average necessary to result in a CRITICAL state\n");
    printf(" <wload15>    = 15 minute load average necessary to result in a WARNING state\n");
    printf(" <cload15>    = 15 minute load average necessary to result in a CRITICAL state\n");
    printf("\n the load average format is the same used by \"uptime\" and \"w\"\n");
    return STATE_UNKNOWN;
  }

  wload1  = atof(argv[1]);
  cload1  = atof(argv[2]);
  wload5  = atof(argv[3]);
  cload5  = atof(argv[4]);
  wload15 = atof(argv[5]);
  cload15 = atof(argv[6]);
  
  if((wload1>cload1)||(wload5>cload5)||(wload15>cload15))
    {
      printf("Inconsistence in parameters: \"warning load\" greater than \"critical load\".\n");
      return STATE_UNKNOWN;
    }

#if defined HAVE_PROC_LOADAVG
  fp=fopen(PROC_LOADAVG,"r");
  if(fp==NULL) 
    {
      printf("Error opening %s\n",PROC_LOADAVG);
      return STATE_UNKNOWN;
    }

  la1 = la5 = la15 = -1;

  while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp))
    {
      tmp_ptr = strtok(input_buffer," ");
      la1     = atof(tmp_ptr);
      tmp_ptr = strtok(NULL," ");
      la5     = atof(tmp_ptr);
      tmp_ptr = strtok(NULL," ");
      la15    = atof(tmp_ptr);
    }

  fclose(fp);
#elif defined HAVE_GETLOADAVG
  result = getloadavg(la,3);
  if (result == -1)
    return STATE_UNKNOWN;
  la1  = la[LOADAVG_1MIN];
  la5  = la[LOADAVG_5MIN];
  la15 = la[LOADAVG_15MIN];
#else
  child_process=spopen(PATH_TO_UPTIME);
  if(child_process==NULL){
      printf("Error opening %s\n",PATH_TO_UPTIME);
      return STATE_UNKNOWN;
  }
  child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
  if(child_stderr==NULL){
    printf("Could not open stderr for %s\n",PATH_TO_UPTIME);
  }
  fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process);
  sscanf(input_buffer,"%*[^l]load average: %f, %f, %f",&la1,&la5,&la15);

  result=spclose(child_process);
  if(result){
      printf("Error code %d returned in %s\n",result,PATH_TO_UPTIME);
      return STATE_UNKNOWN;
  }
#endif

  if((la1 == -1)||(la5 == -1)||(la15 == -1))
    {
      printf("Error processing %s\n",PROC_LOADAVG);
      return STATE_UNKNOWN;
    }
  printf("load average: %.2f, %.2f, %.2f", la1,la5,la15);
  if((la1 >= cload1)||(la5 >= cload5)||(la15 >= cload15))
    {
      printf(" CRITICAL\n");
      return STATE_CRITICAL;
    }
  if((la1 >= wload1)||(la5 >= wload5)||(la15 >= wload15))
    {
      printf(" WARNING\n");
      return STATE_WARNING;
    }
  printf("\n");
  return STATE_OK;
}
