/******************************************************************************
 *
 * CHECK_NNTP.C
 *
 * Program: NNTP plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_NNTP <host_address> [-e expect] [-p port] [-g group] [-wt warn_time] [-ct crit_time] [-to to_sec]
 *
 * Description:
 *
 * This plugin will attempt to open an NNTP connection with the host.
 * Successul connects return STATE_OK, refusals and timeouts return
 * STATE_CRITICAL, other errors return STATE_UNKNOWN.  Successful
 * connects, but incorrect reponse messages from the host result in
 * STATE_WARNING return values.  An invalid newsgroup (if the -g
 * option is used) results in a STATE_WARNING value.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define NNTP_PORT	119
#define NNTP_EXPECT1A   "200"  	/* posting allowed */
#define NNTP_EXPECT1B	"201"	/* posting not allowed */
#define NNTP_EXPECT2	"NNRP"
#define NNTP_QUIT	"QUIT\n"

int process_arguments(int,char **);

int server_port=NNTP_PORT;
char server_expect[MAX_INPUT_BUFFER]=NNTP_EXPECT2;
char server_address[MAX_HOST_ADDRESS_LENGTH]="";
char buffer[MAX_INPUT_BUFFER]="";
char newsgroup[MAX_INPUT_BUFFER]="";
int check_newsgroup=FALSE;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
int verbose=FALSE;


int main(int argc, char **argv){
	int sd;
	int result;
	int recv_result;

	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Invalid command arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("License: GPL\n");
		printf ("\n");
		printf("Usage: %s <host_address> [-e expect] [-p port] [-g group] [-wt warn_time] [-ct crit_time] [-to to_sec] [-v]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [expect]    = String to expect in first line of server response - default is \"%s\"\n",NNTP_EXPECT2);
		printf(" [port]      = Optional port number to use - default is %d\n",NNTP_PORT);
		printf(" [group]     = Newsgroup to check\n");
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin will attempt to contact the NNTP service on the specified host.\n");
		printf("If possible, supply an IP address for the host address, as this will bypass the\n");
		printf("DNS lookup.  All arguments other than the host address are optional.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }


	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	/* try to connect to the host at the given port number */
	time(&start_time);
	result=my_tcp_connect(server_address,server_port,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the NNTP connection string */
		recv_result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);

		/* strip carriage returns from buffer */
		strip(buffer);

		/* return a WARNING status if we couldn't read any data */
		if(recv_result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{

			/* make sure we find the response we are looking for */
			if(!strstr(buffer,NNTP_EXPECT1A) && !strstr(buffer,NNTP_EXPECT1B)){
				printf("Invalid NNTP response received from host");
				result=STATE_WARNING;
			        }
			else if(!strstr(buffer,server_expect)){
				printf("Invalid NNTP header received from host\n");
				result=STATE_WARNING;
			        }

			else if(check_newsgroup==TRUE){

				sprintf(buffer,"GROUP %s\r\n",newsgroup);
				send(sd,buffer,strlen(buffer),0);
				recv_result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);
				if(recv_result==-1)
					strcpy(buffer,"");
				else
					buffer[recv_result]=0;
				strip(buffer);
				if(!strstr(buffer,"211")){
					printf("NNTP problem - newsgroup '%s' not found\n",newsgroup);
					result=STATE_WARNING;
				        }
			        }

			if(result==STATE_OK){

				time(&end_time);

				if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
					result=STATE_CRITICAL;
				else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
					result=STATE_WARNING;

				if(verbose==TRUE)
					printf("NNTP %s - %d sec. response time, %s\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time),buffer);
				else
					printf("NNTP %s - %d second response time\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time));
			        }
		        }

		/* close the connection */
		send(sd,NNTP_QUIT,strlen(NNTP_QUIT),0);
		close(sd);
	        }

	/* reset the alarm timeout */
	alarm(0);

	return result;
        }



/* process arguments passed on the command line */
int process_arguments(int argc, char **argv){
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process all remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the server expect string */
		if(!strcmp(argv[x-1],"-e")){
			if(x<argc){
				strncpy(server_expect,argv[x],sizeof(server_expect)-1);
				server_expect[sizeof(server_expect)-1]=0;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the verbose option */
		else if(!strcmp(argv[x-1],"-v")){
			verbose=TRUE;
		        }

		/* we got the server port number to use */
		else if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the name of the news group to check */
		else if(!strcmp(argv[x-1],"-g")){
			if(x<argc){
				strncpy(newsgroup,argv[x],sizeof(newsgroup)-1);
				newsgroup[sizeof(newsgroup)-1]=0;
				check_newsgroup=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold time */
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold time */
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the socket timeout value */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* else we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }


