/******************************************************************************
 *
 * CHECK_NWSTAT.C
 *
 * Program: NetWare statistics plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_NWSTAT <host_address> [-p port] [-v variable] \
 *                            [-wv warn_value] [-cv crit_value] [-to to_sec]
 *
 * Description:
 *
 * Notes:
 *	- This plugin requres that the MRTGEXT.NLM file distributed
 *	  with James Drews' MRTG extension for NetWare be loaded on the
 *	  Novell servers you wish to check. The extension can be
 *	  downloaded from http://www.engr.wisc.edu/~drews/mrtg/
 *	- Values for critical thresholds should be lower than warning
 *	  thresholds when the * following variables are checked: VPF,
 *	  VKF, LTCH, CBUFF, and LRUM.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define CHECK_NONE	0
#define CHECK_LOAD1  	1
#define CHECK_LOAD5	2
#define CHECK_LOAD15	4
#define CHECK_CONNS	8
#define CHECK_VPF	16
#define CHECK_VKF	32
#define CHECK_LTCH	64
#define CHECK_CBUFF	128
#define CHECK_CDBUFF	256
#define CHECK_LRUM	512

#define NWSTAT_PORT	9999

char server_address[MAX_HOST_ADDRESS_LENGTH];
char volume_name[MAX_INPUT_BUFFER];
int server_port=NWSTAT_PORT;
unsigned long warning_value=0L;
unsigned long critical_value=0L;
int check_warning_value=FALSE;
int check_critical_value=FALSE;
int vars_to_check=CHECK_NONE;

int process_arguments(int,char **);


int main(int argc, char **argv){
	int result;
	char send_buffer[MAX_INPUT_BUFFER];
	char recv_buffer[MAX_INPUT_BUFFER];
	char output_message[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];

	unsigned long free_disk_space;
	unsigned long total_disk_space;
	unsigned long percent_free_space;
	unsigned long current_connections;
	unsigned long utilization;
	unsigned long cache_hits;
	unsigned long cache_buffers;
	unsigned long lru_time;
	char uptime[MAX_INPUT_BUFFER];

	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-v variable] [-wv warn_value] [-cv crit_value] [-to to_sec]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [port]       = Optional port number to use - default is %d\n",server_port);
		printf(" [variable]   = Variable to check.  Valid variables include:\n");
		printf("                  LOAD1    = 1 minute average CPU load\n");
		printf("		  LOAD5	   = 5 minute average CPU load\n");
		printf("		  LOAD15   = 15 minute average CPU load\n");
		printf("                  CONNS    = number of currently licensed connections\n");
		printf("                  VPF<vol> = percent free space on volume <vol>\n");
		printf("	          VKF<vol> = KB of free space on volume <vol>\n");
		printf("                  LTCH     = percent long term cache hits\n");
		printf("                  CBUFF    = current number of cache buffers\n");
		printf("		  CDBUFF   = current number of dirty cache buffers\n");
		printf("		  LRUM     = LRU sitting time in minutes\n");
		printf(" [warn_value] = Threshold for variable value necessary to result in a warning status\n");
		printf(" [crit_value] = Threshold for variable value necessary to result in a critical status\n");
		printf(" [to_sec]     = Number of seconds before connection attempt times out - default is %d sec\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin attempts to contact the MRTGEXT NLM running on a Novell server to gather the\n");
		printf("requested system information.\n");
		printf("\n");
		printf("Notes:\n");
		printf(" - This plugin requres that the MRTGEXT.NLM file distributed with James Drews' MRTG\n");
		printf("   extension for NetWare (available from http://www.engr.wisc.edu/~drews/mrtg/)\n");
		printf("   be loaded on the Novell servers you wish to check.\n");
		printf(" - Critical thresholds should be lower than warning thresholds when the following variables\n");
		printf("   are checked: VPF, VKF, LTCH, CBUFF, and LRUM.\n");
		printf("\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	if(vars_to_check==CHECK_LOAD1 || vars_to_check==CHECK_LOAD5 || vars_to_check==CHECK_LOAD15){
			
		strcpy(send_buffer,"UTIL1\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		utilization=strtoul(recv_buffer,NULL,10);
		strcpy(send_buffer,"UPTIME\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		recv_buffer[strlen(recv_buffer)-1]=0;
		sprintf(uptime,"Up %s,",recv_buffer);

		if(check_critical_value==TRUE && utilization >= critical_value)
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && utilization >= warning_value)
			result=STATE_WARNING;

		switch(vars_to_check){
		case CHECK_LOAD1:
			strcpy(temp_buffer,"1");
			break;
		case CHECK_LOAD5:
			strcpy(temp_buffer,"5");
			break;
		default:
			strcpy(temp_buffer,"15");
			break;
		        }

		sprintf(output_message,"Load %s - %s %s-min load average = %lu%%",(result==STATE_OK)?"ok":"problem",uptime,temp_buffer,utilization);
	        }

	else if(vars_to_check==CHECK_CONNS){

		strcpy(send_buffer,"CONNECT\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		current_connections=strtoul(recv_buffer,NULL,10);

		if(check_critical_value==TRUE && current_connections >= critical_value)
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && current_connections >= warning_value)
			result=STATE_WARNING;
		sprintf(output_message,"Conns %s - %lu current connections",(result==STATE_OK)?"ok":"problem",current_connections);
	        }

	else if(vars_to_check==CHECK_LTCH){

		strcpy(send_buffer,"S1\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		cache_hits=strtoul(recv_buffer,NULL,10);

		if(check_critical_value==TRUE && cache_hits <= critical_value)
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && cache_hits <= warning_value)
			result=STATE_WARNING;
		sprintf(output_message,"Long term cache hits = %lu%%",cache_hits);
	        }

	else if(vars_to_check==CHECK_CBUFF){

		strcpy(send_buffer,"S2\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		cache_buffers=strtoul(recv_buffer,NULL,10);

		if(check_critical_value==TRUE && cache_buffers <= critical_value)
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && cache_buffers <= warning_value)
			result=STATE_WARNING;
		sprintf(output_message,"Total cache buffers = %lu",cache_buffers);
	        }

	else if(vars_to_check==CHECK_CDBUFF){

		strcpy(send_buffer,"S3\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		cache_buffers=strtoul(recv_buffer,NULL,10);

		if(check_critical_value==TRUE && cache_buffers >= critical_value)
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && cache_buffers >= warning_value)
			result=STATE_WARNING;
		sprintf(output_message,"Dirty cache buffers = %lu",cache_buffers);
	        }
			
	else if(vars_to_check==CHECK_LRUM){

		strcpy(send_buffer,"S5\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;
		lru_time=strtoul(recv_buffer,NULL,10);

		if(check_critical_value==TRUE && lru_time <= critical_value)
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && lru_time <= warning_value)
			result=STATE_WARNING;
		sprintf(output_message,"LRU sitting time = %lu minutes",lru_time);
	        }

	else if(vars_to_check==CHECK_VKF){

		sprintf(send_buffer,"VKF%s\r\n",volume_name);
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		if(!strcmp(recv_buffer,"-1\n")){
			sprintf(output_message,"Error: Volume '%s' does not exist!",volume_name);
			result=STATE_CRITICAL;
	                }

		else{
			free_disk_space=strtoul(recv_buffer,NULL,10);
			if(check_critical_value==TRUE && free_disk_space <= critical_value)
				result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && free_disk_space <= warning_value)
				result=STATE_WARNING;
			sprintf(output_message,"%s%lu KB free on volume %s",(result==STATE_OK)?"":"Only ",free_disk_space,volume_name);
		        }
                }

	else if(vars_to_check==CHECK_VPF){

		sprintf(send_buffer,"VKF%s\r\n",volume_name);
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		if(!strcmp(recv_buffer,"-1\n")){
			sprintf(output_message,"Error: Volume '%s' does not exist!",volume_name);
			result=STATE_CRITICAL;
	                }
			
		else{
			free_disk_space=strtoul(recv_buffer,NULL,10);

			sprintf(send_buffer,"VKS%s\r\n",volume_name);
			result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
			if(result!=STATE_OK)
				return result;
			total_disk_space=strtoul(recv_buffer,NULL,10);

			percent_free_space=(unsigned long)(((double)free_disk_space/(double)total_disk_space)*100.0);

			if(check_critical_value==TRUE && percent_free_space <= critical_value)
				result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && percent_free_space <= warning_value)
				result=STATE_WARNING;
			free_disk_space/=1024;
			sprintf(output_message,"%lu MB (%lu%%) free on volume %s",free_disk_space,percent_free_space,volume_name);
		        }
                }

	else{
		strcpy(output_message,"Nothing to check!\n");
		result=STATE_UNKNOWN;
	        }

	/* reset timeout */
	alarm(0);


	printf("%s\n",output_message);

	return result;
        }


/* process command-line arguments */
int process_arguments(int argc, char **argv){
	char buffer[MAX_INPUT_BUFFER];
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the port to use */
		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the variable to check */
		else if(!strcmp(argv[x-1],"-v")){
			if(x<argc){
				
				if(!strcmp(argv[x],"LOAD1"))
					vars_to_check=CHECK_LOAD1;
				else if(!strcmp(argv[x],"LOAD5"))
					vars_to_check=CHECK_LOAD5;
				else if(!strcmp(argv[x],"LOAD15"))
					vars_to_check=CHECK_LOAD15;
				else if(!strcmp(argv[x],"CONNS"))
					vars_to_check=CHECK_CONNS;
				else if(!strcmp(argv[x],"LTCH"))
					vars_to_check=CHECK_LTCH;
				else if(!strcmp(argv[x],"CBUFF"))
					vars_to_check=CHECK_CBUFF;
				else if(!strcmp(argv[x],"CDBUFF"))
					vars_to_check=CHECK_CDBUFF;
				else if(!strcmp(argv[x],"LRUM"))
					vars_to_check=CHECK_LRUM;

				else{

					if(strlen(argv[x])<4)
						return ERROR;
					strncpy(buffer,argv[x],sizeof(buffer)-1);
					buffer[sizeof(buffer)-1]=0;
					buffer[3]=0;
					if(!strcmp(buffer,"VPF")){
						vars_to_check=CHECK_VPF;
						strncpy(volume_name,argv[x]+3,sizeof(volume_name)-1);
						volume_name[sizeof(volume_name)-1]=0;
					        }
					else if(!strcmp(buffer,"VKF")){
						vars_to_check=CHECK_VKF;
						strncpy(volume_name,argv[x]+3,sizeof(volume_name)-1);
						volume_name[sizeof(volume_name)-1]=0;
					        }
					else
						return ERROR;
				        }

				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold for the variable value */
		else if(!strcmp(argv[x-1],"-wv")){
			if(x<argc){
				warning_value=strtoul(argv[x],NULL,10);
				check_warning_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold for the variable value */
		else if(!strcmp(argv[x-1],"-cv")){
			if(x<argc){
				critical_value=strtoul(argv[x],NULL,10);
				check_critical_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the socket timeout */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }
		else
			return ERROR;
	        }

	return OK;
        }


