/******************************************************************************
 *
 * CHECK_OVERCR.C
 *
 * Program: Over-CR collector plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_OVERCR <host_address> [-p port] [-v variable] \
 *                            [-wv warn_value] [-cv crit_value] [-to to_sec]
 *
 * Description:
 *
 * Notes:
 * - This plugin requires that Eric Molitors' Over-CR collector daemon
 *        be running on any UNIX boxes you want to monitor.  Over-CR
 *        is available from * http://www.molitor.org/overcr/
 *
 * Modifications:
 *
 * 08-11-999 Jacob Lundqvist <jaclu@grm.se>
 * Load was presented as a one digit percentage - changed to two digit
 *	value * before load of 11.2 was presented as "1.2%" (not very
 *	high). Warning and Critical params were int's, not very good
 *	for load, changed to doubles, so we can trap loadlimits like
 *	1.5.  Also added more informative LOAD error messages.
 * 
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define CHECK_NONE	0
#define CHECK_LOAD1  	1
#define CHECK_LOAD5	2
#define CHECK_LOAD15	4
#define CHECK_DPU	8
#define CHECK_PROCS	16
#define CHECK_NETSTAT	32
#define CHECK_UPTIME	64

#define OCRSTAT_PORT	2000

char server_address[MAX_HOST_ADDRESS_LENGTH];
int server_port=OCRSTAT_PORT;
double warning_value=0L;
double critical_value=0L;
int check_warning_value=FALSE;
int check_critical_value=FALSE;
int vars_to_check=CHECK_NONE;
int cmd_timeout=1;

int netstat_port=0;
char disk_name[MAX_INPUT_BUFFER];
char process_name[MAX_INPUT_BUFFER];

int process_arguments(int,char **);


int main(int argc, char **argv){
	int result;
	char send_buffer[MAX_INPUT_BUFFER];
	char recv_buffer[MAX_INPUT_BUFFER];
	char output_message[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	char *temp_ptr=NULL;
	int found_disk=FALSE;
	unsigned long percent_used_disk_space=100;
	double load;
	double load_1min;
	double load_5min;
	double load_15min;
	int port_connections=0;
	int processes=0;
	double uptime_raw_hours;
	int uptime_raw_minutes=0;
	int uptime_days=0;
	int uptime_hours=0;
	int uptime_minutes=0;

	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 10-31-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-v variable] [-wv warn_value] [-cv crit_value]\n",argv[0]);
		printf("       [-to to_sec]\n");
		printf("\n");
		printf("Options:\n");
		printf(" [port]        = Optional port number to use - default is %d\n",server_port);
		printf(" [variable]    = Variable to check.  Valid variables include:\n");
		printf("                   LOAD1         = 1 minute average CPU load\n");
		printf("		   LOAD5	        = 5 minute average CPU load\n");
		printf("		   LOAD15        = 15 minute average CPU load\n");
		printf("                   DPU<filesys>  = percent used disk space on filesystem <filesys>\n");
		printf("		   PROC<process> = number of running processes with name <process>\n");
		printf("		   NET<port>     = number of active connections on TCP port <port>\n");
		printf("		   UPTIME        = system uptime in seconds\n");
		printf(" [warn_value]  = Threshold for variable value necessary to result in a warning status\n");
		printf(" [crit_value]  = Threshold for variable value necessary to result in a critical status\n");
		printf(" [to_sec]      = Number of seconds before connection attempt times out - default is %d sec\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin attempts to contact the Over-CR collector daemon running on the remote UNIX server\n");
		printf("in order to gather the requested system information.\n");
		printf("\n");
		printf("Notes:\n");
		printf(" - This plugin requres that Eric Molitors' Over-CR collector daemon be running on the remote\n");
		printf("   server.  Over-CR can be downloaded from http://www.molitor.org/overcr\n");
		printf("   be loaded on the Novell servers you wish to check.\n");
		printf(" - This plugin was tested with version 0.99.53 of the Over-CR collector\n");
		printf(" - For the available options, the critical threshold value should always be higher than the\n");
		printf("   warning threshold value, EXCEPT with the uptime variable (i.e. lower uptimes are worse)!\n");
		printf("\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);
	
	result=STATE_OK;

	if(vars_to_check==CHECK_LOAD1 || vars_to_check==CHECK_LOAD5 || vars_to_check==CHECK_LOAD15){
	
		strcpy(send_buffer,"LOAD\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		temp_ptr=(char *)strtok(recv_buffer,"\r\n");
		if(temp_ptr==NULL){
			printf("Invalid response from server - no load information\n");
			return STATE_CRITICAL;
		        }
		load_1min=strtod(temp_ptr,NULL);
		temp_ptr=(char *)strtok(NULL,"\r\n");
		if(temp_ptr==NULL){
			printf("Invalid response from server after load 1\n");
			return STATE_CRITICAL;
		        }
		load_5min=strtod(temp_ptr,NULL);
		temp_ptr=(char *)strtok(NULL,"\r\n");
		if(temp_ptr==NULL){
			printf("Invalid response from server after load 5\n");
			return STATE_CRITICAL;
		        }
		load_15min=strtod(temp_ptr,NULL);


		switch(vars_to_check){
		case CHECK_LOAD1:
			strcpy(temp_buffer,"1");
			load=load_1min;
			break;
		case CHECK_LOAD5:
			strcpy(temp_buffer,"5");
			load=load_5min;
			break;
		default:
			strcpy(temp_buffer,"15");
			load=load_15min;
			break;
		        }

		if(check_critical_value==TRUE && (load >= critical_value))
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && (load >= warning_value))
			result=STATE_WARNING;
		sprintf(output_message,"Load %s - %s-min load average = %0.2f",(result==STATE_OK)?"ok":"problem",temp_buffer,load);
	        }


	else if(vars_to_check==CHECK_DPU){

		sprintf(send_buffer,"DISKSPACE\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		for(temp_ptr=(char *)strtok(recv_buffer," ");temp_ptr!=NULL;temp_ptr=(char *)strtok(NULL," ")){

			if(!strcmp(temp_ptr,disk_name)){
				found_disk=TRUE;
				temp_ptr=(char *)strtok(NULL,"%");
				if(temp_ptr==NULL){
					printf("Invalid response from server\n");
					return STATE_CRITICAL;
				        }
				percent_used_disk_space=strtoul(temp_ptr,NULL,10);
				break;
			        }

			temp_ptr=(char *)strtok(NULL,"\r\n");
		        }

		/* error if we couldn't find the info for the disk */
		if(found_disk==FALSE){
			sprintf(output_message,"Error: Disk '%s' non-existent or not mounted",disk_name);
			result=STATE_CRITICAL;
		        }

		/* else check the disk space used */
		else{
		
			if(check_critical_value==TRUE && (percent_used_disk_space >= critical_value))
				result=STATE_CRITICAL;
			else if(check_warning_value==TRUE && (percent_used_disk_space >= warning_value))
				result=STATE_WARNING;

			sprintf(output_message,"Disk %s - %lu%% used on %s",(result==STATE_OK)?"ok":"problem",percent_used_disk_space,disk_name);
		        }
                }

	else if(vars_to_check==CHECK_NETSTAT){

		sprintf(send_buffer,"NETSTAT %d\r\n",netstat_port);
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		port_connections=strtod(recv_buffer,NULL);

		if(check_critical_value==TRUE && (port_connections >= critical_value))
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && (port_connections >= warning_value))
			result=STATE_WARNING;

		sprintf(output_message,"Net %s - %d connection%s on port %d",(result==STATE_OK)?"ok":"problem",port_connections,(port_connections==1)?"":"s",netstat_port);
	        }

	else if(vars_to_check==CHECK_PROCS){

		sprintf(send_buffer,"PROCESS %s\r\n",process_name);
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		temp_ptr=(char *)strtok(recv_buffer,"(");	
		if(temp_ptr==NULL){
			printf("Invalid response from server\n");
			return STATE_CRITICAL;
		        }
		temp_ptr=(char *)strtok(NULL,")");
		if(temp_ptr==NULL){
			printf("Invalid response from server\n");
			return STATE_CRITICAL;
		        }
		processes=strtod(temp_ptr,NULL);

		if(check_critical_value==TRUE && (processes >= critical_value))
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && (processes >= warning_value))
			result=STATE_WARNING;

		sprintf(output_message,"Process %s - %d instance%s of %s running",(result==STATE_OK)?"ok":"problem",processes,(processes==1)?"":"s",process_name);
	        }

	else if(vars_to_check==CHECK_UPTIME){
		
		sprintf(send_buffer,"UPTIME\r\n");
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		uptime_raw_hours=strtod(recv_buffer,NULL);
		uptime_raw_minutes=(unsigned long)(uptime_raw_hours * 60.0);

		if(check_critical_value==TRUE && (uptime_raw_minutes <= critical_value))
			result=STATE_CRITICAL;
		else if(check_warning_value==TRUE && (uptime_raw_minutes <= warning_value))
			result=STATE_WARNING;

		uptime_days=uptime_raw_minutes/1440;
		uptime_raw_minutes%=1440;
		uptime_hours=uptime_raw_minutes/60;
		uptime_raw_minutes%=60;
		uptime_minutes=uptime_raw_minutes;

		sprintf(output_message,"Uptime %s - Up %d days %d hours %d minutes",(result==STATE_OK)?"ok":"problem",uptime_days,uptime_hours,uptime_minutes);
                }

	else{
		strcpy(output_message,"Nothing to check!\n");
		result=STATE_UNKNOWN;
	        }

	/* reset timeout */
	alarm(0);

	printf("%s\n",output_message);

	return result;
        }


/* process command-line arguments */
int process_arguments(int argc, char **argv){
	char buffer[MAX_INPUT_BUFFER];
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the port to use */
		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the variable to monitor */
		else if(!strcmp(argv[x-1],"-v")){
			if(x<argc){
				
				if(!strcmp(argv[x],"LOAD1"))
					vars_to_check=CHECK_LOAD1;
				else if(!strcmp(argv[x],"LOAD5"))
					vars_to_check=CHECK_LOAD5;
				else if(!strcmp(argv[x],"LOAD15"))
					vars_to_check=CHECK_LOAD15;
				else if(!strcmp(argv[x],"UPTIME"))
					vars_to_check=CHECK_UPTIME;

				else{

					if(strlen(argv[x])<4)
						return ERROR;

					strcpy(buffer,argv[x]);
					buffer[4]=0;
					if(!strcmp(buffer,"PROC")){
						vars_to_check=CHECK_PROCS;
						strncpy(process_name,argv[x]+4,sizeof(process_name)-1);
						process_name[sizeof(process_name)-1]=0;
					        }

					else{
						strncpy(buffer,argv[x],sizeof(buffer)-1);
						buffer[sizeof(buffer)-1]=0;
						buffer[3]=0;
						if(!strcmp(buffer,"NET")){
							vars_to_check=CHECK_NETSTAT;
							netstat_port=atoi(argv[x]+3);
					                }
						else if(!strcmp(buffer,"DPU")){
							vars_to_check=CHECK_DPU;
							strncpy(disk_name,argv[x]+3,sizeof(disk_name)-1);
							disk_name[sizeof(disk_name)-1]=0;
					                }
						else
							return ERROR;
					        }
				        }

				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold for the variable value */
		else if(!strcmp(argv[x-1],"-wv")){
			if(x<argc){
				warning_value=strtod(argv[x],NULL);			   
			        check_warning_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold for the variable value */
		else if(!strcmp(argv[x-1],"-cv")){
			if(x<argc){
				critical_value=strtod(argv[x],NULL);
				check_critical_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the socket timeout */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
			   
				x++;
			        }
			else
				return ERROR;
		        }

		/* else we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }

