/*****************************************************************************
 *
 * CHECK_REPLY.C
 *
 * Program: TCP Reply plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Jacob Lundqvist (jaclu@grm.se)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_REPLY <host_address> -p port [-s send] -e expect
 *                            [-d] [-ws wait_send] 
 *                            [-wt warn_time] [-ct crit_time] [-to to_sec]
 *
 * Description:
 *
 * This plugin will attempt to open an TCP connection to a specified port
 * with the host, sending an optional string.
 * If the expect string is found in the reply return STATE_OK,
 * refusals and timeouts return STATE_CRITICAL
 * Successful connects, but incorrect reponse messages from the host
 * result in STATE_WARNING return values.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/


#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define DEFAULT_WAIT_SEND 0


int server_port=-1;
char server_address[MAX_HOST_ADDRESS_LENGTH];
char server_send[MAX_INPUT_BUFFER]="";
char server_expect[MAX_INPUT_BUFFER]="";
int wait_send=DEFAULT_WAIT_SEND;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
int dbg_display=0;


int process_arguments(int,char **);


int main(int argc, char **argv){
   int sd;
   int result;
   char buffer[MAX_INPUT_BUFFER];

   result=process_arguments(argc,argv);
   
   if(result!=OK){
      
      printf("Incorrect number of arguments supplied\n");
      printf("\n");
      print_revision(argv[0],"$Revision: 1.4 $");
      printf("Copyright (c) 1999 Jacob Lundqvist (jaclu@grm.se)\n");
      printf("Last Modified: 1999-09-01\n");
      printf("License: GPL\n");
      printf("\n");
      printf("Usage: check_reply <host_address> -p port [-s send] -e expect [-d]\n");
      printf("   [-ws wait_send] [-wt warn_time] [-ct crit_time] [-to to_sec]\n");
      printf("\n");
      printf("Options:\n");
      printf(" [port]      = Port number to use\n");
      printf(" [send]      = Optional string to send to server\n");
      printf(" [expect]    = String to expect in server response\n");
      printf(" -d          = Debug mode, reply is displayed\n");
      printf(" [wait_send] = Wait between send and expect - default is %d seconds\n",DEFAULT_WAIT_SEND);
      printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
      printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
      printf(" [to_sec]    = Number of seconds before connection attempt times out\n");
      printf("                 default is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
      printf("\n");
      printf("This plugin attempts to contact the specified host,\n");
      printf("sending optional send string before checking for response.\n");
      printf("Expect may apeare anywhere in reply.\n");
      printf("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
      printf("\n");
      
      return STATE_UNKNOWN;
   }
   
   /* initialize alarm signal handling */
   signal(SIGALRM,socket_timeout_alarm_handler);
   
   /* set socket timeout */
   alarm(socket_timeout);
   
   /* try to connect to the host at the given port number */
   time(&start_time);
   result=my_tcp_connect(server_address,server_port,&sd);
   
   /* we are connected, so close connection before exiting */
   if(result==STATE_OK){
      
      if (strlen(server_send)>0){
	 /* Something to send? */
	 sprintf(buffer,"%s\n\n",server_send);
	 result=send(sd,buffer,strlen(buffer),0);
      }
	   
      if (wait_send>0){
	 start_time=start_time+wait_send;
	 sleep(wait_send);
      }
      
      /* watch for the expect string */
      result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);
      
      /* return a CRITICAL status if we couldn't read any data */
      if(result==-1){
	 printf("No data received from host\n");
	 result=STATE_CRITICAL;
      }
      
      else{
	 if (dbg_display==1){
	    if (strlen(buffer)>0)
	      printf("%s\n",buffer);
	 }
	 
	 if(!strstr(buffer,server_expect)){
	    printf("Invalid response received from host\n");
	    result=STATE_WARNING;
	 }
	 else{
	    time(&end_time);
	    result=STATE_OK;
	    
	    if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
	      result=STATE_CRITICAL;
	    else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
	      result=STATE_WARNING;
	    
	    printf("%s - %d second response time\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time));
	 }
      }
      
      /* close the connection */
      close(sd);
   }
   
   /* reset the alarm */
   alarm(0);
   
   return result;
}


/* process command-line arguments */
int process_arguments(int argc, char **argv){
   int x;
   
   /* no options were supplied */
   if(argc<2)
     return ERROR;
   
   /* first option is always the server name/address */
   strncpy(server_address,argv[1],sizeof(server_address)-1);
   server_address[sizeof(server_address)-1]=0;
   
   /* process all remaining arguments */
   for(x=3;x<=argc;x++){
      
      /* we got the port number to use */
      if(!strcmp(argv[x-1],"-p")){
	 if(x<argc){
	    server_port=atoi(argv[x]);
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* we got the string to send to the server */
      else if(!strcmp(argv[x-1],"-s")){
	 if(x<argc){
	    strncpy(server_send,argv[x],sizeof(server_send)-1);
	    server_send[sizeof(server_send)-1]=0;
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* we got the string to expect from the server */
      else if(!strcmp(argv[x-1],"-e")){
	 if(x<argc){
	    strncpy(server_expect,argv[x],sizeof(server_expect)-1);
	    server_expect[sizeof(server_expect)-1]=0;
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* we got the wait-to-send */
      else if(!strcmp(argv[x-1],"-ws")){
	 if(x<argc){
	    wait_send=atoi(argv[x]);
	    if(wait_send<=0)
	      return ERROR;
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* Display response */
      else if(!strcmp(argv[x-1],"-d"))
	dbg_display=1;
      
      /* we got the socket timeout */
      else if(!strcmp(argv[x-1],"-to")){
	 if(x<argc){
	    socket_timeout=atoi(argv[x]);
	    if(socket_timeout<=0)
	      return ERROR;
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* we got the warning threshold time */
      else if(!strcmp(argv[x-1],"-wt")){
	 if(x<argc){
	    warning_time=atoi(argv[x]);
	    check_warning_time=TRUE;
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* we got the critical threshold time */
      else if(!strcmp(argv[x-1],"-ct")){
	 if(x<argc){
	    critical_time=atoi(argv[x]);
	    check_critical_time=TRUE;
	    x++;
	 }
	 else
	   return ERROR;
      }
      
      /* else we got something else... */
      else
	return ERROR;
   }
   
   return OK;
}

