/******************************************************************************
 *
 * CHECK_SMTP.C
 *
 * Program: SMTP plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_SMTP <host_address> [-e expect] [-p port] [-to to_sec] [-wt warn_time] [-ct crit_time]
 *
 * Description:
 *
 * This plugin will attempt to open an SMTP connection with the host.
 * Successul connects return STATE_OK, refusals and timeouts return
 * STATE_CRITICAL, other errors return STATE_UNKNOWN.  Successful
 * connects, but incorrect reponse messages from the host result in
 * STATE_WARNING return values.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define SMTP_PORT	25
#define SMTP_EXPECT     "220"
#define SMTP_QUIT	"QUIT\n"

int server_port=SMTP_PORT;
char server_address[MAX_HOST_ADDRESS_LENGTH];
char server_expect[256]=SMTP_EXPECT;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
int verbose=FALSE;

int process_arguments(int,char **);


int main(int argc, char **argv){
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];


	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Incorrect command arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 08-25-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-e expect] [-p port] [-wt warn_time] [-ct crit_time] [-to to_sec] [-v]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [expect]    = String to expect in first line of server response - default is \"%s\"\n",SMTP_EXPECT);
		printf(" [port]      = Optional port number to use - default is %d\n",SMTP_PORT);
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin will attempt to connect with the SMTP service on the specified host.\n");
		printf("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
		printf("\n");

		return STATE_UNKNOWN;
	        }


	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	/* try to connect to the host at the given port number */
	time(&start_time);
	result=my_tcp_connect(server_address,server_port,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the SMTP connection string */
		result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);

		/* strip the buffer of carriage returns */
		strip(buffer);

		/* return a WARNING status if we couldn't read any data */
		if(result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{

			/* make sure we find the response we are looking for */
			if(!strstr(buffer,server_expect)){
				if(server_port==SMTP_PORT)
					printf("Invalid SMTP response received from host\n");
				else
					printf("Invalid SMTP response received from host on port %d\n",server_port);
				result=STATE_WARNING;
			        }

			else{

				time(&end_time);

				result=STATE_OK;

				if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
					result=STATE_CRITICAL;
				else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
					result=STATE_WARNING;

				if(verbose==TRUE)
					printf("SMTP %s - %d sec. response time, %s\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time),buffer);
				else
					printf("SMTP %s - %d second response time\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time));
			        }
		        }

		/* close the connection */
		send(sd,SMTP_QUIT,strlen(SMTP_QUIT),0);
		close(sd);
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



int process_arguments(int argc, char **argv){
	int x;


	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process all remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the string to expect from the server */
		if(!strcmp(argv[x-1],"-e")){
			if(x<argc){
				strncpy(server_expect,argv[x],sizeof(server_expect)-1);
				server_expect[sizeof(server_expect)-1]=0;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the port number to connect to */
		else if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the verbose option */
		else if(!strcmp(argv[x-1],"-v")){
			verbose=TRUE;
		        }

		/* we got the socket timeout */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold time */
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold time */
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }
