/******************************************************************************
 *
 * CHECK_SNMP.C
 *
 * Program: SNMP plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/06/13 14:12:09 $
 *
 * Description:
 *
 * This plugin uses the 'snmpget' command included with the UCD-SNMP
 * package.  If you don't have the package installed you will need to
 * download it from http://ucd-snmp.ucdavis.edu before you can use
 * this plugin.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *./plugins/check_snmp 127.0.0.1 -c public -o .1.3.6.1.4.1.2021.9.1.2.1
 *****************************************************************************/

#include "../common/common.h"
#include "utils.h"
#include "popen.h"

#define mark(a) ((a)!=0?"*":"")

#define CHECK_UNDEF 0
#define CRIT_PRESENT 1
#define CRIT_STRING 2
#define CRIT_REGEX 4
#define CRIT_GT 8
#define CRIT_LT 16
#define CRIT_GE 32
#define CRIT_LE 64
#define CRIT_EQ 128
#define CRIT_NE 256
#define CRIT_RANGE 512
#define WARN_PRESENT 1024
#define WARN_STRING 2048
#define WARN_REGEX 4096
#define WARN_GT 8192
#define WARN_LT 16384
#define WARN_GE 32768
#define WARN_LE 65536
#define WARN_EQ 131072
#define WARN_NE 262144
#define WARN_RANGE 524288

#define MAX_OIDS 8
#define MAX_DELIM_LENGTH 8
#define DEFAULT_DELIMITER "="
#define DEFAULT_OUTPUT_DELIMITER " "

void print_usage(char *);
void print_help(char *);
int process_arguments(int,char **);
int call_getopt(int,char **);
int check_num(int);
char* clarify_message(char *);
int lu_getll(unsigned long *, char *);
int lu_getul(unsigned long *, char *);

#ifdef HAVE_REGEX_H
#include <regex.h>
char regex_expect[MAX_INPUT_BUFFER]="";
regex_t preg;
regmatch_t pmatch[10];
char timestamp[10]="";
char regex[MAX_INPUT_BUFFER];
char errbuf[MAX_INPUT_BUFFER];
int cflags=REG_EXTENDED|REG_NOSUB|REG_NEWLINE;
int eflags=0;
int errcode,excode;
#endif

char server_address[MAX_HOST_ADDRESS_LENGTH]="";
char community[MAX_INPUT_BUFFER]="public";
char oid[MAX_INPUT_BUFFER]="";
char label[MAX_INPUT_BUFFER]="";
char *labels[MAX_OIDS];
char units[MAX_INPUT_BUFFER]="";
char *unitv[MAX_OIDS];
char string_value[MAX_INPUT_BUFFER]="";
int nlabels;
int nunits;
unsigned long lower_warn_lim[MAX_OIDS];
unsigned long upper_warn_lim[MAX_OIDS];
unsigned long lower_crit_lim[MAX_OIDS];
unsigned long upper_crit_lim[MAX_OIDS];
unsigned long response_value[MAX_OIDS];
int check_warning_value=FALSE;
int check_critical_value=FALSE;
int eval_method[MAX_OIDS];
char delimiter[MAX_DELIM_LENGTH]=DEFAULT_DELIMITER;
char output_delim[MAX_DELIM_LENGTH]=DEFAULT_OUTPUT_DELIMITER;


int main(int argc, char **argv)
{
	int i=0;
	int loop_counter=0;
	int iresult=STATE_UNKNOWN;
	int found=0;
	int method;
	int result=STATE_DEPENDENT;
	char command_line[MAX_INPUT_BUFFER];
	char input_buffer[MAX_INPUT_BUFFER];
	char response[MAX_INPUT_BUFFER];
	char temp_str[MAX_INPUT_BUFFER];
	char outbuff[MAX_INPUT_BUFFER]="";
	char *temp_ptr;

	if (process_arguments(argc,argv)==ERROR) {
		printf("Incorrect arguments supplied\n");
		print_revision(my_basename(argv[0]),"$Revision: 1.12 $");
		print_usage(my_basename(argv[0]));
		return STATE_UNKNOWN;
	}

	/* create the command line to execute */
	sprintf(command_line,"%s -m ALL -v 1 %s %s %s",
					PATH_TO_SNMPGET,
					server_address,
					community,
					oid);

	/* run the command */
	child_process=spopen(command_line);
	if (child_process==NULL) {
		printf("Could not open pipe: %s\n",command_line);
		exit(STATE_UNKNOWN);
	}

	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL) {
		printf("Could not open stderr for %s\n",command_line);
	}

	while (fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process)) {

		found++;

		/* strip the newline character from the end of the input */
		if(input_buffer[strlen(input_buffer)-1]=='\n')
			input_buffer[strlen(input_buffer)-1]=0;

		temp_ptr=(char *)strstr(input_buffer,delimiter);
		if (!temp_ptr) {
			printf("%s problem - %s\n",(strcmp(label,""))?label:"SNMP",input_buffer);
			result=STATE_CRITICAL;
			break;
		}

			/* scan response until we hit a digit (unless we're looking for string) */
		method=eval_method[i];
		if(!(method & CRIT_STRING ||
				 method & CRIT_REGEX ||
				 method & CRIT_PRESENT ||
				 method & WARN_STRING ||
				 method & WARN_REGEX ||
				 method & WARN_PRESENT)) {
			while(!isdigit(*temp_ptr) && temp_ptr)
				temp_ptr++;
		} else {	/* else skip the first space after the delimiter */
			temp_ptr++;
			while(isspace(*temp_ptr) && temp_ptr)
				temp_ptr++;
		}

		/* copy remainder of the returned buffer and keep that as the reponse */
		if(temp_ptr==NULL)
			strcpy(response,"");
		else
			strncpy(response,temp_ptr,MAX_INPUT_BUFFER);

		if (eval_method[i] & CRIT_PRESENT) {
			iresult=STATE_OK;
		}
		else if (eval_method[i] & CRIT_STRING) {
			if(strcmp(response,string_value))
				iresult=STATE_CRITICAL;
			else
				iresult=STATE_OK;
			printf("%s\n",response);
		}
		else if (eval_method[i] & CRIT_REGEX) {
#ifdef HAVE_REGEX_H
			excode=regexec(&preg,response,10,pmatch,eflags);
			if(excode==0) {
				iresult=STATE_OK;
			}	else if (excode!=REG_NOMATCH) {
				regerror(excode,&preg,errbuf,MAX_INPUT_BUFFER);
				printf ("Execute Error: %s\n",errbuf);
				exit(STATE_CRITICAL);
			} else {
				iresult=STATE_CRITICAL;
			}
#else
			printf("SNMP UNKNOWN: call for regex which was not a compiled option");
			exit(STATE_UNKNOWN);
#endif
		} else {
			response_value[i]=strtoul(response,NULL,10);
			iresult=check_num(i);
		}
		result=max(result,iresult);
		strcpy(temp_str,outbuff);
		if(nlabels>1&&loop_counter<=nlabels)
			snprintf(outbuff,MAX_INPUT_BUFFER,"%s%s%s %s%s%s",temp_str,output_delim,labels[loop_counter++],mark(iresult),response,mark(iresult));
		else
			snprintf(outbuff,MAX_INPUT_BUFFER,"%s%s%s%s%s",temp_str,output_delim,mark(iresult),response,mark(iresult));
		if( nunits>0&&loop_counter<=nunits) {
			strcpy(temp_str,outbuff);
			snprintf(outbuff,MAX_INPUT_BUFFER,"%s %s",temp_str,unitv[i]);
		}
		i++;
	} /* end while */

	if (!found) {
		printf("%s problem - No data recieved from host\nCMD: %s\n",(!strcmp(label,""))?"SNMP":label,command_line);
		exit(STATE_UNKNOWN);
	}

	/* WARNING if output found on stderr */
	if(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		result=max(result,STATE_WARNING);

	/* close stderr */
	(void)fclose(child_stderr);

	/* close the pipe */
	if(spclose(child_process))
		result=max(result,STATE_WARNING);

	printf("%s %s -%s %s\n",(!strcmp(label,""))?"SNMP":label,state_text(result),outbuff,(nunits>0?"":units));
	return result;
}

/* process command-line arguments */
int process_arguments(int argc, char **argv)
{
	int c;

	if(argc<2)
		return ERROR;

	for (c=1; c<argc; c++) {
		if(strcmp("-to",argv[c])==0)
			strcpy(argv[c],"-t");
		if(strcmp("-wv",argv[c])==0)
			strcpy(argv[c],"-w");
		if(strcmp("-cv",argv[c])==0)
			strcpy(argv[c],"-c");
	}

	c=0;
	while (c+=(call_getopt(argc-c,&argv[c]))) {
		if (argc <= c)
			break;
		if (server_address[0]==0) {
			strncpy(server_address,argv[c],sizeof(server_address)-1);
			server_address[sizeof(server_address)-1]=0;
		}
	}

	return c;
}

int call_getopt(int argc, char **argv)
{
	char* ptr;
	int c,i=1;
	int j=0,jj=0;

#ifdef HAVE_GETOPT_H
	int option_index = 0;
	static struct option long_options[] =
	{ 
		{"help",             no_argument,       0, 'h'},
		{"version",          no_argument,       0, 'V'},
		{"timeout",          required_argument, 0, 't'},
		{"critical",         required_argument, 0, 'c'},
		{"warning",          required_argument, 0, 'w'},
		{"hostname",         required_argument, 0, 'H'},
		{"community",        required_argument, 0, 'C'},
		{"oid",              required_argument, 0, 'o'},
		{"object",           required_argument, 0, 'o'},
		{"delimiter",        required_argument, 0, 'd'},
		{"output-delimiter", required_argument, 0, 'D'},
		{"string",           required_argument, 0, 's'},
		{"regex",            required_argument, 0, 'r'},
		{"ereg",             required_argument, 0, 'r'},
		{"eregi",            required_argument, 0, 'R'},
		{"label",            required_argument, 0, 'l'},
		{"units",            required_argument, 0, 'u'},
		{0,0,0,0}
	};
#endif

	while (1) {
#ifdef HAVE_GETOPT_H
		c = getopt_long(argc,argv,"+?hVt:c:w:H:C:o:d:D:s:R:r:l:u:",long_options,&option_index);
#else
		c = getopt(argc,argv,"+?hVt:c:w:H:C:o:d:D:s:R:r:l:u:");
#endif

		if (c==-1||c==EOF)
			break;

		switch (c)
			{
			case 't':
			case 'c':
			case 'w':
			case 'H':
			case 'C':
			case 'o':
			case 'd':
			case 'D':
			case 's':
			case 'R':
			case 'r':
			case 'l':
			case 'u':
				i++;
			}

		switch (c)
			{
			case '?': /* help */
				printf("%s: Unknown argument: %s\n\n",my_basename(argv[0]),optarg);
				print_usage(my_basename(argv[0]));
				exit(STATE_UNKNOWN);
			case 'h': /* help */
				print_help(my_basename(argv[0]));
				exit(STATE_UNKNOWN);
			case 'V': /* version */
				print_revision(my_basename(argv[0]),"$Revision: 1.12 $");
				exit(STATE_UNKNOWN);
			case 't': /* timeout period */
				if (!is_integer(optarg)) {
					printf("%s: Timeout Interval must be an integer!\n\n",my_basename(argv[0]));
					print_usage(my_basename(argv[0]));
					exit(STATE_UNKNOWN);
				}
				timeout_interval = atoi(optarg);
				break;
			case 'c': /* critical time threshold */
				if (strspn(optarg,"0123456789:,") < strlen(optarg)) {
					printf("Invalid critical threshold: %s\n",optarg);
					print_usage(my_basename(argv[0]));
					exit(STATE_UNKNOWN);
				}
				for (ptr=optarg,jj=0; ptr&&jj<MAX_OIDS; jj++) {
					if (lu_getll(&lower_crit_lim[jj],ptr)==1)
						eval_method[jj] |= CRIT_LT;
					if (lu_getul(&upper_crit_lim[jj],ptr)==1)
						eval_method[jj] |= CRIT_GT;
					(ptr=index(ptr,','))?ptr++:ptr;
				}
				break;
			case 'w': /* warning time threshold */
				if (strspn(optarg,"0123456789:,") < strlen(optarg)) {
					printf("Invalid warning threshold: %s\n",optarg);
					print_usage(my_basename(argv[0]));
					exit(STATE_UNKNOWN);
				}
				for (ptr=optarg,jj=0; ptr&&jj<MAX_OIDS; jj++) {
					if(lu_getll(&lower_warn_lim[jj],ptr)==1)
						eval_method[jj] |= WARN_LT;
					if(lu_getul(&upper_warn_lim[jj],ptr)==1)
						eval_method[jj] |= WARN_GT;
					(ptr=index(ptr,','))?ptr++:ptr;
				}
				break;
			case 'H': /* Host or server */
				strncpy(server_address,optarg,sizeof(server_address)-1);
				server_address[sizeof(server_address)-1]=0;
				break;
			case 'C': /* group or community */
				strncpy(community,optarg,sizeof(community)-1);
				community[sizeof(community)-1]=0;
				break;
			case 'o': /* object identifier */
				for(ptr=optarg;(ptr=index(ptr,','));ptr++)
					ptr[0]=' ';
				strncpy(oid,optarg,sizeof(oid)-1);
				oid[sizeof(oid)-1]=0;
				for(ptr=optarg,j=1;(ptr=index(ptr,' '));ptr++)
					j++;
				break;
			case 'd': /* delimiter */
				strncpy(delimiter,optarg,MAX_DELIM_LENGTH-1);
				delimiter[MAX_DELIM_LENGTH-1]=0;
				break;
			case 'D': /* output-delimiter */
				strncpy(output_delim,optarg,MAX_DELIM_LENGTH-1);
				output_delim[MAX_DELIM_LENGTH-1]=0;
				break;
			case 's': /* string or substring */
				strncpy(string_value,optarg,sizeof(string_value)-1);
				string_value[sizeof(string_value)-1]=0;
				eval_method[jj++]=CRIT_STRING;
				break;
			case 'R': /* regex */
#ifdef HAVE_REGEX_H
				cflags=REG_ICASE;
#endif
			case 'r': /* regex */
#ifdef HAVE_REGEX_H
				cflags|=REG_EXTENDED|REG_NOSUB|REG_NEWLINE;
				strncpy(regex_expect,optarg,sizeof(regex_expect)-1);
				regex_expect[sizeof(regex_expect)-1]=0;
				errcode=regcomp(&preg,regex_expect,cflags);
				if(errcode!=0){
					regerror(errcode,&preg,errbuf,MAX_INPUT_BUFFER);
					printf("Could Not Compile Regular Expression");
					return ERROR;
				}
				eval_method[jj++]=CRIT_REGEX;
#else
				printf("SNMP UNKNOWN: call for regex which was not a compiled option");
				exit(STATE_UNKNOWN);
#endif
				break;
			case 'l': /* label */
				strncpy(label,optarg,MAX_INPUT_BUFFER-1);
				label[MAX_INPUT_BUFFER-1]=0;
				labels[0]=label;
				for(nlabels=0;(ptr=index(labels[nlabels],','));labels[++nlabels]=++ptr)
					ptr[0]=0;
				break;
			case 'u': /* units */
				strncpy(units,optarg,MAX_INPUT_BUFFER-1);
				units[MAX_INPUT_BUFFER-1]=0;
				unitv[0]=units;
				for(nunits=0;(ptr=index(unitv[nunits],','));unitv[++nunits]=++ptr)
					ptr[0]=0;
				break;
			}
	}
	return i;
}

void print_usage(char *cmd){
	printf
		("Usage: check_snmp -H <ip_address> -o <OID> [-w warn_range] [-c crit_range] \n"
		 "                  [-C community] [-s string] [-r regex] [-R regexi] [-t timeout]\n"
		 "                  [-l label] [-u units] [-d delimiter] [-D output-delimiter]\n"
		 "       check_snmp --help\n"
		 "       check_snmp --version\n");
}

void print_help(char *cmd){
	printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n"
				 "License: GPL\n\n");
	print_usage(cmd);
	printf
		("\nOptions:\n"
		 " -h, --help\n"
		 "    Print detailed help screen\n"
		 " -V, --version\n"
		 "    Print version information\n"
		 " -H, --hostname=HOST\n"
		 "    Name or IP address of the device you wish to query\n"
		 " -o, --oid=OID(s)\n"
		 "    Object identifier(s) whose value you wish to query\n"
		 " -w, --warning=INTEGER_RANGE(s)\n"
		 "    Range(s) which will not result in a WARNING status\n"
		 " -c, --critical=INTEGER_RANGE(s)\n"
		 "    Range(s) which will not result in a CRITICAL status\n"
		 " -C, --community=STRING\n"
		 "    Optional community string for SNMP communication\n"
		 "    (default is \"public\")\n"
		 " -u, --units=STRING\n"
		 "    Units label(s) for output data (e.g., 'sec.').\n"
		 " -d, --delimiter=STRING\n"
		 "    Delimiter to use when parsing returned data. Default is \"%s\"\n"
		 "    Any data on the right hand side of the delimiter is considered\n"
		 "    to be the data that should be used in the evaluation.\n"
		 " -t, --timeout=INTEGER\n"
		 "    Seconds to wait before plugin times out (see also netsaint server timeout)\n"
		 " -D, --output-delimiter=STRING\n"
		 "    Separates output on multiple OID requests\n"
		 " -s, --string=STRING\n"
		 "    Return OK state (for that OID) if STRING matches\n"
		 " -r, --ereg=REGEX\n"
		 "    Return OK state (for that OID) if extended regular expression REGEX matches\n"
		 " -R, --eregi=REGEX\n"
		 "    Return OK state (for that OID) if case-insensitive extended REGEX matches\n"
		 " -l, --label=STRING\n"
		 "    Prefix label for output from plugin (default -s 'SNMP')\n\n"
		 "- This plugin uses the 'snmpget' command included with the UCD-SNMP package.\n"
		 "  If you don't have the package installed, you will need to download it from\n"
		 "  http://ucd-snmp.ucdavis.edu before you can use this plugin.\n"
		 "- Multiple OIDs may be indicated by a comma- or space-delimited list (lists with\n"
		 "  internal spaces must be quoted)\n"
		 "- Ranges are inclusive and are indicated with colons. When specified as\n"
		 "  'min:max' a STATE_OK will be returned if the result is within the indicated\n"
		 "  range or is equal to the upper or lower bound. A non-OK state will be\n"
		 "  returned if the result is outside the specified range.\n"
		 "- If spcified in the order 'max:min' a non-OK state will be returned if the\n"
		 "  result is within the (inclusive) range.\n"
		 "- Upper or lower bounds may be omitted to skip checking the respective limit.\n"
		 "- Bare integers are interpreted as upper limits.\n"
		 "- When checking multiple OIDs, separate ranges by commas like '-w 1:10,1:,:20'\n"
		 "- All evaluation methods other than PR, STR, and SUBSTR expect that the value\n"
		 "  returned from the SNMP query is an unsigned integer.\n\n",
		 DEFAULT_DELIMITER);
}

char* clarify_message(char *msg){
	int i = 0;
	int foo;
	char tmpmsg_c[MAX_INPUT_BUFFER];
	char *tmpmsg = (char*)&tmpmsg_c;
	tmpmsg = strcpy(tmpmsg,msg);
	if (! strncmp(tmpmsg," Hex:",5)) {
		tmpmsg=strtok(tmpmsg,":");
		while((tmpmsg=strtok(NULL," "))) {
			foo = strtol(tmpmsg,NULL,16);
			/* Translate chars that are not the same value in the printers
			 * character set.
			 */
			switch(foo) {
			case 208:
				{
					foo = 197;
					break;
				}
			case 216:
				{
					foo = 196;
					break;
				}
			}
			msg[i] = foo;
			i++;
		}
		msg[i] = 0;
	}
	return(msg);
}


int check_num(int i){
	int result;
	/*		printf("%d %d %lu %lu %lu %lu %lu\n",i,eval_method[i],response_value[i],upper_crit_lim[i],upper_warn_lim[i],lower_crit_lim[i],lower_warn_lim[i]); */
	result=STATE_OK;
	if(eval_method[i] & WARN_GT && eval_method[i] & WARN_LT &&
		 lower_warn_lim[i] > upper_warn_lim[i]) {
		if(response_value[i] <= lower_warn_lim[i] &&
			 response_value[i] >= upper_warn_lim[i]) {
			result=STATE_WARNING;
		}
	}
	else if 
		((eval_method[i] & WARN_GT && response_value[i]> upper_warn_lim[i]) ||
		 (eval_method[i] & WARN_GE && response_value[i]>=upper_warn_lim[i]) ||
		 (eval_method[i] & WARN_LT && response_value[i]< lower_warn_lim[i]) ||
		 (eval_method[i] & WARN_LE && response_value[i]<=lower_warn_lim[i]) ||
		 (eval_method[i] & WARN_EQ && response_value[i]==upper_warn_lim[i]) ||
		 (eval_method[i] & WARN_NE && response_value[i]!=upper_warn_lim[i])) {
		result=STATE_WARNING;
	}

	if(eval_method[i] & CRIT_GT && eval_method[i] & CRIT_LT &&
		 lower_warn_lim[i] > upper_warn_lim[i]){
		if(response_value[i] <= lower_crit_lim[i] &&
			 response_value[i] >= upper_crit_lim[i]){
			result=STATE_CRITICAL;
		}
	}
	else if
		((eval_method[i] & CRIT_GT && response_value[i]> upper_crit_lim[i]) ||
		 (eval_method[i] & CRIT_GE && response_value[i]>=upper_crit_lim[i]) ||
		 (eval_method[i] & CRIT_LT && response_value[i]< lower_crit_lim[i]) ||
		 (eval_method[i] & CRIT_LE && response_value[i]<=lower_crit_lim[i]) ||
		 (eval_method[i] & CRIT_EQ && response_value[i]==upper_crit_lim[i]) ||
		 (eval_method[i] & CRIT_NE && response_value[i]!=upper_crit_lim[i])){
		result=STATE_CRITICAL;
	}

	return result;
}


int lu_getll(unsigned long *ll, char *str){
	char tmp[100];
	if(strchr(str,':')==NULL)
		return 0;
	if(strchr(str,',')!=NULL&&(strchr(str,',')<strchr(str,':')))
		return 0;
	if(sscanf(str,"%lu%[:]",ll,tmp)==2)
		return 1;
	return 0;
}

int lu_getul(unsigned long *ul, char *str){
	char tmp[100];
	if(sscanf(str,"%lu%[^,]",ul,tmp)==1)
		return 1;
	if(sscanf(str,":%lu%[^,]",ul,tmp)==1)
		return 1;
	if(sscanf(str,"%*u:%lu%[^,]",ul,tmp)==1)
		return 1;
	return 0;
}
