/******************************************************************************
 *
 * CHECK_SWAP.C
 *
 * Program: Process plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)
 *
 * Last Modified: $Date: 2000/06/02 19:13:05 $
 *
 * Command line: check_swap <wsize> <csize> [] [] [command_name]
 *
 * Description:
 *
 * This plugin will check for processes whose total image size exceeds the 
 * warning or critical thresholds given on the command line.  With no
 * command_name, everything that shows up on ps is evaluated.  Otherwise,
 * only jobs with the command_name given are examined.  This program is
 * particularly useful if you have to run a piece of commercial software
 * that has a memory leak.  With it you can shut down ad restart the processes
 * whenever the program threatens to take over your system.
 *
 * Modifications:
 *
 * 08-18-1999 Ethan Galstad (netsaint@linuxbox.com)
 *	      Changed code to use common include file, changed fclose() to
 *	      pclose().
 * 09-03-1999 Karl DeBisschop
 *            use safe popen, look at absolute bytes free as well
 *
 *****************************************************************************/

#include "../common/common.h"
#include "popen.h"
#include "utils.h"

int main(int argc, char **argv){
	int wswap, cswap, pct, wsize=0, csize=0;
	int tswap, uswap, fswap;
	int result=STATE_OK;
	char input_buffer[MAX_INPUT_BUFFER];
#ifdef HAVE_SWAP
	char *temp_buffer;
#endif
#ifdef HAVE_PROC_MEMINFO
	FILE *fp;
#endif
	char str[32];
	char status[MAX_INPUT_BUFFER];

	if(argc!=3 && argc!=5){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.6 $");
		printf("Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)\n");
		printf("Last Modified: 18 Aug 1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wswap> <cswap> [wsize] [csize]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <wswap>     = Consumed swap percentage necessary to cause a WARNING state\n");
		printf(" <cswap>     = Consumed swap percentage necessary to cause a CRITICAL state\n");
		printf(" [wsize]     = Remaining bytes swap necessary to cause a WARNING state\n");
		printf(" [csize]     = Remaining bytes swap necessary to cause a CRITICAL state\n");
		printf("\n");
		printf("This plugin will check all of the swap partitions and return an\n");
		printf("error if the number is above either of the thresholds given.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the number of procs use as warning and critical thresholds */
	wswap=atoi(argv[1]);
	cswap=atoi(argv[2]);
	if(argc == 5){
	  wsize=atoi(argv[3]);
	  csize=atoi(argv[4]);
	}


#ifdef HAVE_PROC_MEMINFO
	fp=fopen(PROC_MEMINFO,"r");
	sprintf(status,"%s","Swap used:");
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)) {
	  sscanf(input_buffer, " %s %d %d %d", str, &tswap, &uswap, &fswap);
	  if (strstr(str,"Swap")){
	    pct = 100 * (((float) uswap)/((float) tswap));
	    sprintf(status,"%s %2d%% (%d bytes out of %d)",status,pct,uswap,tswap);
	    if (pct >= cswap || fswap <= csize)
	      result = STATE_CRITICAL;
	    else if (pct >= wswap || fswap <= wsize)
	      result = max(result, STATE_WARNING);
	    break;
	  }
	}
	fclose(fp);
#else
#ifdef HAVE_SWAP
	child_process=spopen(SWAP_COMMAND);
	if(child_process==NULL){
		printf("Could not open pipe: %s\n",SWAP_COMMAND);
		return STATE_UNKNOWN;
	        }

	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL)
		printf("Could not open stderr for %s\n",SWAP_COMMAND);
	
	sprintf(str, "%s", "");
	/* read 1st line */
	fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process); 
	if(strcmp(SWAP_FORMAT,"")==0){
	  temp_buffer=strtok(input_buffer," \n");
	  while (temp_buffer) {
	    if (strstr(temp_buffer,"blocks"))
	      sprintf(str,"%s %s",str,"%f");
	    else if (strstr(temp_buffer,"free"))
	      sprintf(str,"%s %s",str,"%f");
	    else
	      sprintf(str,"%s %s",str,"%*s");
	    temp_buffer=strtok(NULL," \n");
	  }
	}

	sprintf(status,"%s","Swap used:");
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process)) {
	  sscanf(input_buffer, SWAP_FORMAT, &tswap, &fswap);
	  uswap = tswap - fswap;
	  pct = 100 * ((float) uswap)/((float) tswap);
	  sprintf(status,"%s %2d%% (%d bytes out of %d)",status,pct,uswap,tswap);
	  if (pct >= cswap || fswap <= csize)
	    result = STATE_CRITICAL;
	  else if (pct >= wswap || fswap <= wsize)
	    result = max(result,STATE_WARNING);
	}

	/* If we get anything on STDERR, at least set warning */
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		result=max(result,STATE_WARNING);

	/* close stderr */
	(void)fclose(child_stderr);

	/* close the pipe */
	if(spclose(child_process))
		result=max(result,STATE_WARNING);
#endif
#endif

#ifndef SWAP_COMMAND	
#ifndef SWAP_FILE
	return STATE_UNKNOWN;
#endif
#endif

	if(result==STATE_OK)
	  printf("Swap ok - %s\n",status);
	else if(result==STATE_CRITICAL)
	  printf("CRITICAL - %s\n",status);
	else if(result==STATE_WARNING)
	  printf("WARNING - %s\n",status);
	else if(result==STATE_UNKNOWN)
	  printf("Unable to read output\n");
	else {
	  result = STATE_UNKNOWN;
	  printf("UNKNOWN - %s\n",status);
	}

	return result;
        }
