/******************************************************************************
 *
 * CHECK_TCP.C
 *
 * Program: TCP port plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_TCP <host_address> [-p port] [-to to_sec] [-wt warn_time] [-ct crit_time]
 *
 * Description:
 *
 * This plugin will attempt to connect to the specified port
 * on the host.  Successul connects return STATE_OK, refusals
 * and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

time_t start_time,end_time;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
int verbose=FALSE;

int process_arguments(int,char **);

int server_port=0;
char server_address[MAX_HOST_ADDRESS_LENGTH];


int main(int argc, char **argv){
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];

	result=process_arguments(argc,argv);

	if(result!=OK){

		printf("Invalid command arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-wt warn_time] [-ct crit_time] [-to to_sec] [-v]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [port]      = Port number to connect to\n");
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin will attempt to connect to a TCP port on the specified host.\n");
		printf("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
		printf("\n");

		return STATE_UNKNOWN;
	        }


	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	/* try to connect to the host at the given port number */
	time(&start_time);
	result=my_tcp_connect(server_address,server_port,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		time(&end_time);

		if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
			result=STATE_CRITICAL;
		else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
			result=STATE_WARNING;

		if(verbose==TRUE){
			result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);
			if(result==-1)
				strcpy(buffer,"");
			else
				strip(buffer);
			printf("Connection %s on port %d - %d sec. response time, %s\n",(result==STATE_OK)?"accepted":"problem",server_port,(int)(end_time-start_time),buffer);
		        }
		else
			printf("Connection %s on port %d - %d second response time\n",(result==STATE_OK)?"accepted":"problem",server_port,(int)(end_time-start_time));

		/* close the connection */
		close(sd);
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



/* process all arguments passed on the command line */
int process_arguments(int argc, char **argv){
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the port number to use */
		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the verbose option */
		else if(!strcmp(argv[x-1],"-v")){
			verbose=TRUE;
		        }

		/* we got the socket timeout to use */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning time threshold */
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical time threshold */
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* else we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }
