/******************************************************************************
 *
 * CHECK_TIME.C
 *
 * Program: Network time server plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_TIME <host_address> [-p port] [-wd warn_diff] [-cd crit_diff] \ 
 *			    [-to to_sec] [-wt warn_time] [-ct crit_time]
 *
 * Description:
 *
 * This plugin will attempt to connect to the specified port
 * on the host.  Successul connects return STATE_OK, refusals
 * and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

#define TIME_PORT	37

#define UNIX_EPOCH	2208988800UL

unsigned long server_time, raw_server_time;
time_t diff_time;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
unsigned long warning_diff=0;
int check_warning_diff=FALSE;
unsigned long critical_diff=0;
int check_critical_diff=FALSE;
int server_port=TIME_PORT;
char server_address[MAX_HOST_ADDRESS_LENGTH];


int process_arguments(int,char **);


int main(int argc, char **argv){
	int sd;
	int result;

	if(process_arguments(argc,argv)!=OK){

		printf("Invalid command arguments supplied\n\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n"
		       "License: GPL\n\n"
		       "Usage: %s <host_address> [-p port] [-wd warn_diff] [-cd crit_diff] [-wt warn_time] [-ct crit_time] [-to to_sec]\n\n"
		       "Options:\n"
		       " [port]      = Port number to connect to - default is %d\n"
		       " [warn_diff] = Time difference in seconds necessary to result in a warning status\n"
		       " [crit_diff] = Time difference in seconds necessary to result in a critical status\n"
		       " [warn_time] = Response time in seconds necessary to result in a warning status\n"
		       " [crit_time] = Response time in seconds necessary to result in a critical status\n"
		       " [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n"
		       "\n"
		       "This plugin will attempt to connect to a TIME port on the specified host.\n"
		       "If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n\n",
		       argv[0],TIME_PORT,DEFAULT_SOCKET_TIMEOUT);

		return STATE_UNKNOWN;
	        }


	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	/* try to connect to the host at the given port number */
	time(&start_time);
	result=my_tcp_connect(server_address,server_port,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the FTP connection string */
		result=recv(sd,&raw_server_time,sizeof(raw_server_time),0);

		/* return a WARNING status if we couldn't read any data */
		if(result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{

			time(&end_time);

			result=STATE_OK;

			if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
				result=STATE_CRITICAL;
			else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
				result=STATE_WARNING;

			if(result!=STATE_OK)
				printf("Time server problem - %d second response time\n",(int)(end_time-start_time));

			else{

			        server_time=ntohl(raw_server_time)-UNIX_EPOCH;

				if(server_time>end_time)
					diff_time=server_time-end_time;
				else
					diff_time=end_time-server_time;

				if(check_critical_diff==TRUE && diff_time>critical_diff)
					result=STATE_CRITICAL;
				else if(check_warning_diff==TRUE && diff_time>warning_diff)
					result=STATE_WARNING;

				printf("Time server %s - %lu sec. time difference\n",(result==STATE_OK)?"ok":"problem",diff_time);
		                }

		        }

		/* close the connection */
		close(sd);
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



/* process all command-line arguments */
int process_arguments(int argc, char **argv){
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the port to connect to */
		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the socket timeout */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold time */
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold time */
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning time difference threshold */
		else if(!strcmp(argv[x-1],"-wd")){
			if(x<argc){
				warning_diff=strtoul(argv[x],NULL,10);
				check_warning_diff=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical time difference threshold */
		else if(!strcmp(argv[x-1],"-cd")){
			if(x<argc){
				critical_diff=strtoul(argv[x],NULL,10);
				check_critical_diff=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* else we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }
