/******************************************************************************
 *
 * CHECK_UDP.C
 *
 * Program: UDP port plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/01/28 20:03:56 $
 *
 * Command line: CHECK_UDP <host_address> [-p port] [-s send] [-e expect] \
 *			   [-wt warn_time] [-ct crit_time] [-to to_sec]
 *
 * Description:
 *
 * This plugin will attempt to connect to the specified port
 * on the host.  Successul connects return STATE_OK, refusals
 * and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"
#include "utils.h"

int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;

int process_arguments(int,char **);

int server_port=0;
char server_address[MAX_HOST_ADDRESS_LENGTH];
char server_send[MAX_INPUT_BUFFER]="";
char server_expect[MAX_INPUT_BUFFER]="";


int main(int argc, char **argv){
	int result;
	char recv_buffer[MAX_INPUT_BUFFER];

	result=process_arguments(argc,argv);

	if(result!=OK){

		printf("Invalid command arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.4 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-s send] [-e expect] [-wt warn_time] [-ct crit_time] [-to to_sec]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [port]      = Port number to connect to\n");
		printf(" [send]      = Optional string to send to the server when initiating the connection\n");
		printf(" [expect]    = Substring that is expected to be present in the server response\n");
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin will attempt to connect to a UDP port on the specified host.\n");
		printf("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
		printf("\n");

		return STATE_UNKNOWN;
	        }


	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);

	time(&start_time);
	result=process_udp_request(server_address,server_port,server_send,recv_buffer,sizeof(recv_buffer));
	time(&end_time);

	if(result!=STATE_OK){
		printf("No response from host on port %d\n",server_port);
		result=STATE_CRITICAL;
                }

	else{

		/* check to see if we got the response we wanted */
		if(strcmp(server_expect,"")){
			if(!strstr(recv_buffer,server_expect)){
				printf("Invalid response received from host on port %d\n",server_port);
				result=STATE_CRITICAL;
			        }
		        }
	        }

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
			result=STATE_CRITICAL;
		else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
			result=STATE_WARNING;

		printf("Connection %s on port %d - %d second response time\n",(result==STATE_OK)?"accepted":"problem",server_port,(int)(end_time-start_time));
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



/* process command-line arguments */
int process_arguments(int argc, char **argv){
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]=0;

	/* process remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the port number */
		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the socket timeout */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold time */
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold time */
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the string to expect from the server */
		else if(!strcmp(argv[x-1],"-e")){
			if(x<argc){
				if(strlen(argv[x])<MAX_INPUT_BUFFER)
					strcpy(server_expect,argv[x]);
				else
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the string to send to the server */
		else if(!strcmp(argv[x-1],"-s")){
			if(x<argc){
				if(strlen(argv[x])<MAX_INPUT_BUFFER)
					strcpy(server_send,argv[x]);
				else
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }

		/* else we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }



