/******************************************************************************
 *
 * CHECK_USERS.C
 *
 * Program: Current users plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: $Date: 2000/05/03 03:11:02 $
 * Modifications: 
 *
 * 1999-11-17 Karl DeBisschop
 *  - check stderr and status from spoen/spclose
 *  - reformat commenst to fit 80-cahr screen
 *  - set default result to STATE_UNKNOWN
 *  - initialize users at -1, eliminate 'found' variable
 *
 * Command line: CHECK_USERS <wusers> <cusers>
 *
 * Description:
 *
 * This plugin will use the /usr/bin/who command to check the number
 * of users currently logged into the system.  If number of logged in
 * user exceeds the number specified by the <cusers> option, a
 * STATE_CRITICAL is return.  It it exceeds <wusers>, a STATE_WARNING
 * is returned.  Errors reading the output from the who command result
 * in a STATE_UNKNOWN error.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/common.h"
#include "popen.h"
#include "utils.h"

#define possibly_set(a,b) ((a) == 0 ? (b) : 0)

int main(int argc, char **argv){
	int wusers;
	int cusers;
	int users=-1;
	int result=STATE_UNKNOWN;
	char input_buffer[MAX_INPUT_BUFFER];

	if(argc!=3){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		print_revision(argv[0],"$Revision: 1.7 $");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wusers> <cusers>\n",argv[0]);
		printf("\n");
		printf("Note:\n");
		printf(" <wusers> = Number of logged in users necessary to result in a WARNING state\n");
		printf(" <cusers> = Number of logged in users necessary to result in a CRITICAL state\n");
		printf("\n");
		printf("This plugin will check the number of users currently logged in on the local\n");
		printf("system and generate an error if the number exceeds the thresholds specified.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the warning and critical thresholds */
	wusers=atoi(argv[1]);
	cusers=atoi(argv[2]);

	/* run the command */
	child_process=spopen(WHO_COMMAND);
	if(child_process==NULL){
		printf("Could not open pipe: %s\n",WHO_COMMAND);
		return STATE_UNKNOWN;
	        }

	child_stderr=fdopen(child_stderr_array[fileno(child_process)],"r");
	if(child_stderr==NULL)
		printf("Could not open stderr for %s\n",WHO_COMMAND);

	users=0;

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_process)){

		/* increment 'users' on all lines except total user count */
		if(input_buffer[0]!='#'){
			users++;
			continue;
			}

		/* get total logged in users */
		if(sscanf(input_buffer,"# users=%d",&users)==1)
			break;

		}

	/* check STDERR */
	if(fgets(input_buffer,MAX_INPUT_BUFFER-1,child_stderr))
		result=possibly_set(result,STATE_UNKNOWN);
	(void)fclose(child_stderr);

	/* close the pipe */
	if(spclose(child_process))
		result=possibly_set(result,STATE_UNKNOWN);

	/* else check the user count against warning and critical thresholds */
	if(users>=cusers)
		result=STATE_CRITICAL;
	else if(users>=wusers)
		result=STATE_WARNING;
	else if(users>=0)
		result=STATE_OK;
	
	if(result==STATE_OK)
		printf("Users ok - %d users logged in\n",users);
	else if(result==STATE_UNKNOWN)
		printf("Unable to read output\n");
	else
		printf("%d users currently logged in\n",users);

	return result;
        }
