/*****************************************************************************
 *
 * utils.c
 *
 * Library of useful functions for plugins
 *
 * Copyright (c) 2000 Karl DeBisschop (karl@debisschop.net)
 * License: GPL
 *
 * $Revision: 1.14 $
 * $Date: 2000/06/25 07:26:04 $
 ****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "../common/version.h"
#include <stdarg.h>

extern int timeout_interval;

char *my_basename(char *);
void print_revision(char *,const char *);
void terminate (int, const char *fmt, ...);
RETSIGTYPE timeout_alarm_handler(int);

int is_host(char *);
int is_dotted_quad(char *);
int is_hostname(char *);

int is_integer(char *);
int is_intpos(char *);
int is_intneg(char *);
int is_intnonneg(char *);
int is_intpercent(char *);

int is_numeric(char *);
int is_positive(char *);
int is_negative(char *);
int is_nonnegative(char *);
int is_percentage(char *);

int is_option(char *);

#define LABELLEN 63
#define STRLEN 64

char *my_basename(char *path)
{
  if(!strstr(path,"/"))
    return path;
  else
    return 1+rindex(path,'/');
}


void print_revision(char *command_name, const char *revision_string)
{
	char *plugin_name;
	char plugin_revision[STRLEN];
	char this_release[STRLEN]=PACKAGE_VERSION;

	plugin_name=my_basename(command_name);
	if(sscanf(revision_string,"$Revision: %[0-9.]",plugin_revision)!=1)
	  strncpy(plugin_revision,"N/A",STRLEN);
	printf("%s (netsaint-plugins %s) %s\n",
				 plugin_name,
				 this_release,
				 plugin_revision);
	printf("The netsaint plugins come with ABSOLUTELY NO WARRANTY. You may redistribute\n"
				 "copies of the plugins under the terms of the GNU General Public License.\n"
				 "For more information about these matters, see the file named COPYING.\n");

}


void terminate (int result, const char *fmt, ...)
{
	va_list ap;
	va_start(ap,fmt);
	vprintf(fmt,ap);
	va_end(ap);
	exit(result);
}

void timeout_alarm_handler(int signo)
{
  if(signo==SIGALRM){
    printf("CRITICAL - Plugin timed out after %d seconds\n",timeout_interval);
    exit(STATE_CRITICAL);
  }
}

int is_host(char *address)
{
	if(is_dotted_quad(address)||is_hostname(address))
		return(TRUE);
	return(FALSE);
}

int is_dotted_quad(char *address)
{
	int o1, o2, o3, o4;
	char c[1];

	if(sscanf(address,"%d.%d.%d.%d%c",&o1,&o2,&o3,&o4,c)!=4)
		return FALSE;
	else if(o1>255 || o2>255 || o3>255 || o4>255)
		return FALSE;
	else if(o1<0 || o2<0 || o3<0 || o4<0)
		return FALSE;
	else
		return TRUE;
	}

/* from RFC-1035
 * 
 * The labels must follow the rules for ARPANET host names.  They must
 * start with a letter, end with a letter or digit, and have as interior
 * characters only letters, digits, and hyphen.  There are also some
 * restrictions on the length.  Labels must be 63 characters or less. */

int is_hostname(char *s1)
{
	char token[LABELLEN+1];
	char *s2;

	if(strlen(s1)>63)
		return FALSE;
	if(sscanf(s1,"%*[a-zA-Z0-9.-]%[^a-zA-Z0-9.-]",token)==1)
		return FALSE;
	if(sscanf(s1,"%[^a-zA-Z]",token)==1)
	  return FALSE;
	while((s2=index(s1,'.'))){
	  s1=s2;
	  s1++;
	  if(sscanf(s1,"%[^a-zA-Z]",token)==1)
	    return FALSE;
	}
	return TRUE;
}

int is_numeric(char *number)
{
	char tmp[1];
	float x;
	if(sscanf(number,"%f%c",&x,tmp)==1)
		return(TRUE);
	return(FALSE);
}

int is_positive(char *number)
{
	if(is_numeric(number) && atof(number)>0.0)
		return(TRUE);
	return(FALSE);
}

int is_negative(char *number)
{
	if(is_numeric(number) && atof(number)<0.0)
		return(TRUE);
	return(FALSE);
}

int is_nonnegative(char *number)
{
	if(is_numeric(number) && atof(number)>=0.0)
		return(TRUE);
	return(FALSE);
}

int is_percentage(char *number)
{
	int x;
	if(is_numeric(number) && (x=atof(number))>=0 && x<=100)
		return(TRUE);
	return(FALSE);
}

int is_integer(char *number)
{
	char tmp[1];
	int n;
	if(sscanf(number,"%d%c",&n,tmp)==1)
		return(TRUE);
	return(FALSE);
}

int is_intpos(char *number)
{
	if(is_integer(number) && atoi(number)>0.0)
		return(TRUE);
	return(FALSE);
}

int is_intneg(char *number)
{
	if(is_integer(number) && atoi(number)<0.0)
		return(TRUE);
	return(FALSE);
}

int is_intnonneg(char *number)
{
	if(is_integer(number) && atoi(number)>=0.0)
		return(TRUE);
	return(FALSE);
}

int is_intpercent(char *number)
{
	int i;
	if(is_integer(number) && (i=atoi(number))>=0 && i<=100)
		return(TRUE);
	return(FALSE);
}

int is_option(char *opt)
{
	if (strspn(opt,"-")==1||strspn(opt,"-")==2)
		return TRUE;
	return FALSE;
}
